# -*- coding: utf-8 -*-
from odin.mapping import helpers


class TestMappingHelpers(object):
    def test_sum_fields(self):
        assert 42, helpers.sum_fields(22 == 20)

    def test_cat_fields(self):
        assert "abcd", helpers.cat_fields()("ab" == "cd")
        assert "ab-cd", helpers.cat_fields('-')("ab" == "cd")

    def test_split_field(self):
        assert ["ab", "cd"] == helpers.split_field()("ab cd")
        assert ["abcd"] == helpers.split_field('-')("abcd")
        assert ["ab", "cd"] == helpers.split_field('-')("ab-cd")
        assert ["ab", "cd", "ef"] == helpers.split_field('-')("ab-cd-ef")
        assert ["ab", "cd-ef"], helpers.split_field('-' == 1)("ab-cd-ef")



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function
from odin import exceptions
from odin.fields import ScalarField
from odin.validators import EMPTY_VALUES
from .datatypes import Amount

__all__ = ('AmountField', )


class AmountField(ScalarField):
    """
    Field that contains a monetary amount (with an optional currency).
    """
    default_error_messages = {
        'invalid': "'%s' value must be a (amount, currency).",
        'invalid_currency': "'%s' currency is not supported.",
    }
    data_type_name = "Amount"

    def __init__(self, allowed_currencies=None, **kwargs):
        super(AmountField, self).__init__(**kwargs)
        self.allowed_currencies = allowed_currencies

    def to_python(self, value):
        if value in EMPTY_VALUES:
            return
        if isinstance(value, Amount):
            return value

        try:
            return Amount(value)
        except (ValueError, TypeError):
            msg = self.error_messages['invalid'] % value
            raise exceptions.ValidationError(msg)

    def validate(self, value):
        super(AmountField, self).validate(value)
        if self.allowed_currencies and value not in EMPTY_VALUES:
            if value.currency not in self.allowed_currencies:
                msg = self.error_messages['invalid_currency'] % value.currency
                raise exceptions.ValidationError(msg)

    def prepare(self, value):
        if value in EMPTY_VALUES:
            return
        return float(value), value.currency.code



from __future__ import absolute_import, print_function
import argparse
import sys

from .resource import ResourceSummary
from .mapping import MappingSummary


def import_item(item):
    module, obj = item.rsplit('.', 1)
    module = __import__(module, globals(), locals(), [obj])
    return getattr(module, obj)


def get_options():
    parser = argparse.ArgumentParser(prog="odin.contrib.inspect")
    main_subparsers = parser.add_subparsers(dest='command')

    # Resource ##################################

    subparser = main_subparsers.add_parser('resource', help='Inspect a Odin resource')
    subparser.add_argument('RESOURCE', help='Path to resource')

    # Mapping ###################################

    subparser = main_subparsers.add_parser('mapping', help='Inspect a Odin mapping')
    subparser.add_argument('MAPPING', help='Path to mapping')

    return parser.parse_args()


def main():
    options = get_options()

    if options.command == 'resource':
        resource = import_item(options.RESOURCE)
        summary = ResourceSummary(resource, sys.stdout)
        summary.render()
    elif options.command == 'mapping':
        mapping = import_item(options.MAPPING)
        summary = MappingSummary(mapping, sys.stdout)
        summary.render()
    else:
        print("No command supplied.", file=sys.stdout)
        exit(1)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
from odin import exceptions


class TestValidationException(object):
    def test_with_string(self):
        test_message = "Test message"
        target = exceptions.ValidationError(test_message)

        assert [test_message] == target.messages
        assert not hasattr(target, 'message_dict')
        assert "['Test message']" == str(target)
        assert "ValidationError(['Test message'])" == repr(target)

    def test_with_list(self):
        test_message_list = ["Test message", "Test message 2"]
        target = exceptions.ValidationError(test_message_list)

        assert test_message_list == target.messages
        assert not hasattr(target, 'message_dict')
        assert "['Test message', 'Test message 2']" == str(target)
        assert "ValidationError(['Test message', 'Test message 2'])" == repr(target)

    def test_with_dict(self):
        test_message_dict = {
            "Test Key 1": ["Test Message 1"],
            "Test Key 2": ["Test Message 2"],
        }
        target = exceptions.ValidationError(test_message_dict)

        assert test_message_dict == target.message_dict
        assert [{
            "Test Key 1": ["Test Message 1"],
            "Test Key 2": ["Test Message 2"],
        }] == target.messages

        expected = {'Test Key 2': ['Test Message 2'], 'Test Key 1': ['Test Message 1']}
        assert expected == target.message_dict
        assert(
            "ValidationError({'Test Key 1': ['Test Message 1'], 'Test Key 2': ['Test Message 2']})" ==
            repr(target)
        )



# -*- coding: utf-8 -*-
from copy import deepcopy
import pytest
import datetime
from odin.fields import *
from odin.fields import Field, TimeStampField, NOT_PROVIDED
from odin.datetimeutil import utc, FixedTimezone
from odin.fields.virtual import VirtualField
from odin.validators import MinValueValidator, MaxValueValidator, MaxLengthValidator, RegexValidator
from odin.exceptions import ValidationError


class ObjectValue(object):
    pass


class TestValidator(object):
    message = 'Default message'
    code = 'test_code'

    def __init__(self, fail, *params):
        self.fail = fail
        self.params = params

    def __call__(self, value):
        if self.fail:
            raise ValidationError(code=self.code, message=self.message, params=self.params)


class FieldTest(Field):
    def to_python(self, value):
        return value


class TestDynamicTypeNameField(IntegerField):
    @staticmethod
    def data_type_name(instance):
        return "Foo"


class TestField(object):
    def test_error_messages_no_overrides(self):
        target = FieldTest()

        assert {
            'invalid_choice': 'Value %r is not a valid choice.',
            'null': 'This field cannot be null.',
            'required': 'This field is required.',
        } == target.error_messages

    def test_error_messages_override_add(self):
        target = FieldTest(error_messages={
            'null': 'Override',
            'other': 'Other Value',
        })

        assert {
            'invalid_choice': 'Value %r is not a valid choice.',
            'null': 'Override',
            'required': 'This field is required.',
            'other': 'Other Value',
        } == target.error_messages

    def test_set_attributes_from_name(self):
        target = FieldTest()
        target.set_attributes_from_name("test_name")

        assert "test_name" == target.name
        assert "test_name" == target.attname
        assert "test name" == target.verbose_name
        assert "test names" == target.verbose_name_plural

    def test_set_attributes_from_name_with_name(self):
        target = FieldTest(name="init_name")
        target.set_attributes_from_name("test_name")

        assert "init_name" == target.name
        assert "test_name" == target.attname
        assert "init name" == target.verbose_name
        assert "init names" == target.verbose_name_plural

    def test_set_attributes_from_name_with_verbose_name(self):
        target = FieldTest(verbose_name="init Verbose Name")
        target.set_attributes_from_name("test_name")

        assert "test_name" == target.name
        assert "test_name" == target.attname
        assert "init Verbose Name" == target.verbose_name
        assert "init Verbose Names" == target.verbose_name_plural

    def test_has_default(self):
        target = FieldTest()

        assert not target.has_default()

    def test_has_default_supplied(self):
        target = FieldTest(default="123")

        assert target.has_default()

    def test_get_default(self):
        target = FieldTest()

        assert target.get_default() is None

    def test_get_default_supplied(self):
        target = FieldTest(default="123")

        assert "123" == target.get_default()

    def test_get_default_callable(self):
        target = FieldTest(default=lambda: "321")

        assert "321" == target.get_default()

    def test_value_from_object(self):
        target = FieldTest()
        target.set_attributes_from_name("test_name")

        an_obj = ObjectValue()
        setattr(an_obj, "test_name", "test_value")

        actual = target.value_from_object(an_obj)
        assert "test_value" == actual

    def test__repr(self):
        target = Field()
        assert "<odin.fields.Field>" == repr(target)
        target.set_attributes_from_name("eek")
        assert "<odin.fields.Field: eek>" == repr(target)

    def test__deep_copy(self):
        field = FieldTest(name="Test")
        target_copy = deepcopy(field)
        target_assign = field
        assert field is target_assign
        assert field is not target_copy
        assert field.name == target_copy.name

    def test_run_validators_and_override_validator_message(self):
        target = FieldTest(error_messages={'test_code': 'Override message'}, validators=[TestValidator(True)])

        try:
            target.run_validators("Placeholder")
        except ValidationError as ve:
            assert 'Override message' == ve.messages[0]
        else:
            raise AssertionError("Validation Error not raised.")

    def test_run_validators_and_override_validator_message_with_params(self):
        target = FieldTest(error_messages={'test_code': 'Override message: %s'}, validators=[TestValidator(True, "123")])

        try:
            target.run_validators("Placeholder")
        except ValidationError as ve:
            assert ['Override message: 123'] == ve.messages
        else:
            raise AssertionError("Validation Error not raised.")

    def test_default_to_python_raises_not_implemented(self):
        target = Field()
        pytest.raises(NotImplementedError, target.to_python, "Anything...")

    def test_clean_uses_default_if_value_is_not_provided_is_true(self):
        target = FieldTest(use_default_if_not_provided=True, default='foo')
        actual = target.clean(NOT_PROVIDED)
        assert 'foo' == actual

    def test_clean_uses_default_if_value_is_not_provided_is_false(self):
        # Need to allow None as the if use_default_if_not_provided is false NOT_PROVIDED evaluates to None.
        target = FieldTest(use_default_if_not_provided=False, default='foo', null=True)
        actual = target.clean(NOT_PROVIDED)
        assert actual is None


class VirtualFieldTest(VirtualField):
    pass


class TestVirtualField(object):
    def test_creation_counter(self):
        current_count = VirtualField.creation_counter
        next_count = current_count + 1
        target = VirtualField()

        assert VirtualField.creation_counter == next_count
        assert target.creation_counter == current_count
        assert hash(target) == current_count

    def test_repr(self):
        target = VirtualFieldTest()
        assert "<tests.test_fields.VirtualFieldTest>" == repr(target)
        target.set_attributes_from_name("eek")
        assert "<tests.test_fields.VirtualFieldTest: eek>" == repr(target)

    def test_default_descriptor_behaviour(self):
        class TestObj(object):
            test_field = VirtualField()
        target = TestObj()

        with pytest.raises(NotImplementedError):
            _ = target.test_field

        with pytest.raises(AttributeError) as excinfo:
            target.test_field = 123

        assert "Read only" == str(excinfo.value)


class TestFields(object):
    def assertValidatorIn(self, validatorClass, validators):
        """
        Assert that the specified validator is in the validation list.
        :param validatorClass:
        :param validators:
        """
        for v in validators:
            if isinstance(v, validatorClass):
                return
        raise AssertionError("Validator %r was not found in list of validators." % validatorClass)

    def assertValidatorNotIn(self, validatorClass, validators):
        """
        Assert that the specified validator is not in the validation list.
        :param validatorClass:
        :param validators:
        """
        for v in validators:
            if isinstance(v, validatorClass):
                raise AssertionError("Validator %r was found in list of validators." % validatorClass)

    # BooleanField ############################################################

    def test_booleanfield_1(self):
        f = BooleanField()
        pytest.raises(ValidationError, f.clean, None)
        pytest.raises(ValidationError, f.clean, '')
        assert f.clean(True)
        assert f.clean(1)
        assert f.clean('Yes')
        assert f.clean('true')
        assert f.clean('T')
        assert f.clean('1')
        pytest.raises(ValidationError, f.clean, 'Awesome!')
        assert not f.clean(False)
        assert not f.clean(0)
        assert not f.clean('No')
        assert not f.clean('false')
        assert not f.clean('F')
        assert not f.clean('0')

    # StringField #############################################################

    def test_stringfield_1(self):
        f = StringField()
        assert '1' == f.clean('1')
        assert 'eek' == f.clean('eek')
        pytest.raises(ValidationError, f.clean, None)
        assert '[1, 2, 3]', f.clean([1, 2 == 3])
        assert f.max_length == None
        self.assertValidatorNotIn(MaxLengthValidator, f.validators)

    def test_stringfield_2(self):
        f = StringField(null=True)
        assert '1' == f.clean('1')
        assert 'eek' == f.clean('eek')
        assert None == f.clean(None)
        assert '[1, 2, 3]', f.clean([1, 2 == 3])
        assert f.max_length == None
        self.assertValidatorNotIn(MaxLengthValidator, f.validators)

    def test_stringfield_3(self):
        f = StringField(null=True, max_length=10)
        assert None == f.clean(None)
        assert '' == f.clean('')
        assert '12345' == f.clean('12345')
        assert '1234567890' == f.clean('1234567890')
        pytest.raises(ValidationError, f.clean, '1234567890a')
        assert f.max_length == 10
        self.assertValidatorIn(MaxLengthValidator, f.validators)

    # URLField ################################################################

    def test_urlfield_1(self):
        f = UrlField()
        assert 'http://www.github.com' == f.clean('http://www.github.com')
        pytest.raises(ValidationError, f.clean, 'eek')
        pytest.raises(ValidationError, f.clean, None)
        assert f.max_length == None
        self.assertValidatorIn(RegexValidator, f.validators)

    # IntegerField ############################################################

    def test_integerfield_1(self):
        f = IntegerField()
        pytest.raises(ValidationError, f.clean, None)
        pytest.raises(ValidationError, f.clean, 'abc')
        assert 123 == f.clean(123)
        assert 123 == f.clean('123')
        assert 123 == f.clean(123.5)
        assert None == f.min_value
        self.assertValidatorNotIn(MinValueValidator, f.validators)
        assert None == f.max_value
        self.assertValidatorNotIn(MaxValueValidator, f.validators)

    def test_integerfield_2(self):
        f = IntegerField(null=True)
        assert None == f.clean(None)
        pytest.raises(ValidationError, f.clean, 'abc')
        assert 69 == f.clean(69)
        assert 69 == f.clean('69')
        assert 69 == f.clean(69.5)
        assert None == f.min_value
        self.assertValidatorNotIn(MinValueValidator, f.validators)
        assert None == f.max_value
        self.assertValidatorNotIn(MaxValueValidator, f.validators)

    def test_integerfield_3(self):
        f = IntegerField(min_value=50, max_value=100)
        pytest.raises(ValidationError, f.clean, None)
        pytest.raises(ValidationError, f.clean, 'abc')
        assert 69 == f.clean(69)
        assert 69 == f.clean('69')
        assert 69 == f.clean(69.5)
        assert 50 == f.clean(50)
        assert 100 == f.clean(100)
        pytest.raises(ValidationError, f.clean, 30)
        pytest.raises(ValidationError, f.clean, 110)
        assert 50 == f.min_value
        self.assertValidatorIn(MinValueValidator, f.validators)
        assert 100 == f.max_value
        self.assertValidatorIn(MaxValueValidator, f.validators)

    # FloatField ##############################################################

    def test_floatfield_1(self):
        f = FloatField()
        pytest.raises(ValidationError, f.clean, None)
        pytest.raises(ValidationError, f.clean, 'abc')
        assert 123 == f.clean(123)
        assert 123.5 == f.clean('123.5')
        assert 123.5 == f.clean(123.5)
        assert None == f.min_value
        self.assertValidatorNotIn(MinValueValidator, f.validators)
        assert None == f.max_value
        self.assertValidatorNotIn(MaxValueValidator, f.validators)

    def test_floatfield_2(self):
        f = FloatField(null=True)
        assert None == f.clean(None)
        pytest.raises(ValidationError, f.clean, 'abc')
        assert 69 == f.clean(69)
        assert 69.5 == f.clean('69.5')
        assert 69.5 == f.clean(69.5)
        assert None == f.min_value
        self.assertValidatorNotIn(MinValueValidator, f.validators)
        assert None == f.max_value
        self.assertValidatorNotIn(MaxValueValidator, f.validators)

    def test_floatfield_3(self):
        f = FloatField(min_value=50.5, max_value=100.4)
        pytest.raises(ValidationError, f.clean, None)
        pytest.raises(ValidationError, f.clean, 'abc')
        assert 69 == f.clean(69)
        assert 69.5 == f.clean('69.5')
        assert 69.5 == f.clean(69.5)
        assert 50.5 == f.clean(50.5)
        assert 100.4 == f.clean(100.4)
        pytest.raises(ValidationError, f.clean, 30)
        pytest.raises(ValidationError, f.clean, 110)
        assert 50.5 == f.min_value
        self.assertValidatorIn(MinValueValidator, f.validators)
        assert 100.4 == f.max_value
        self.assertValidatorIn(MaxValueValidator, f.validators)

    # DateField ###############################################################

    def test_datefield_1(self):
        f = DateField()
        pytest.raises(ValidationError, f.clean, None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        assert datetime.date(2013, 11, 24) == f.clean('2013-11-24')
        assert datetime.date(2013, 11, 24) == f.clean(datetime.date(2013, 11, 24))
        assert datetime.date(2013, 11, 24) == f.clean(datetime.datetime(2013, 11, 24, 1, 14))

    def test_datefield_2(self):
        f = DateField(null=True)
        assert None == f.clean(None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        assert datetime.date(2013, 11, 24) == f.clean('2013-11-24')
        assert datetime.date(2013, 11, 24) == f.clean(datetime.date(2013, 11, 24))
        assert datetime.date(2013, 11, 24) == f.clean(datetime.datetime(2013, 11, 24, 1, 14))

    # TimeField ###############################################################

    def test_timefield_1(self):
        f = TimeField(assume_local=False)
        pytest.raises(ValidationError, f.clean, None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        assert datetime.time(18, 43, tzinfo=utc) == f.clean('18:43:00.000Z')
        assert datetime.time(18, 43, tzinfo=utc) == f.clean(datetime.time(18, 43, tzinfo=utc))

    def test_timefield_2(self):
        f = TimeField(assume_local=False, null=True)
        assert None == f.clean(None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        assert datetime.time(18, 43, tzinfo=utc) == f.clean('18:43:00.000Z')
        assert datetime.time(18, 43, tzinfo=utc) == f.clean(datetime.time(18, 43, tzinfo=utc))

    # DateTimeField ###########################################################

    def test_datetimefield_1(self):
        f = DateTimeField(assume_local=False)
        pytest.raises(ValidationError, f.clean, None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        assert datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc) == f.clean('2013-11-24T18:43:00.000Z')
        assert datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc) == f.clean('2013-11-24 18:43:00.000Z')
        assert datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc) == f.clean(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc))

    def test_datetimefield_2(self):
        f = DateTimeField(assume_local=False, null=True)
        assert None == f.clean(None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        assert datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc) == f.clean('2013-11-24T18:43:00.000Z')
        assert datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc) == f.clean('2013-11-24 18:43:00.000Z')
        assert datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc) == f.clean(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc))

    # HttpDateTimeField #######################################################

    def test_httpdatetimefield_1(self):
        f = HttpDateTimeField()
        pytest.raises(ValidationError, f.clean, None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        assert datetime.datetime(2012, 8, 29, 17, 12, 58, tzinfo=utc) == f.clean('Wed Aug 29 17:12:58 +0000 2012')
        assert datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc) == f.clean(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc))

    def test_httpdatetimefield_2(self):
        f = HttpDateTimeField(null=True)
        assert None == f.clean(None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        assert datetime.datetime(2012, 8, 29, 17, 12, 58, tzinfo=utc) == f.clean('Wed Aug 29 17:12:58 +0000 2012')
        assert datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc) == f.clean(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc))

    # TimeStampField ##########################################################

    def test_timestampfield_1(self):
        f = TimeStampField()
        pytest.raises(ValidationError, f.clean, None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 'Wed Aug 29 17:12:58 +0000 2012')
        assert datetime.datetime(1970, 1, 1, 0, 2, 3, tzinfo=utc) == f.clean(123)
        assert datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc) == f.clean(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc))

    def test_timestampfield_2(self):
        f = TimeStampField(null=True)
        assert None == f.clean(None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 'Wed Aug 29 17:12:58 +0000 2012')
        assert datetime.datetime(1970, 1, 1, 0, 2, 3, tzinfo=utc) == f.clean(123)
        assert datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc) == f.clean(datetime.datetime(2013, 11, 24, 18, 43, tzinfo=utc))

    def test_timestampfield_3(self):
        f = TimeStampField()
        assert None == f.prepare(None)
        assert 123 == f.prepare(datetime.datetime(1970, 1, 1, 0, 2, 3, tzinfo=utc))
        assert 123 == f.prepare(123)
        assert 123 == f.prepare(
            datetime.datetime(1970, 1, 1, 10, 2, 3, tzinfo=FixedTimezone.from_hours_minutes(10)))

    # DictField ###############################################################

    def test_dictfield_1(self):
        f = DictField()
        pytest.raises(ValidationError, f.clean, None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        assert {} == f.clean({})
        assert {'foo': 'bar'} == f.clean({'foo': 'bar'})
        assert f.default == dict

    def test_dictfield_2(self):
        f = DictField(null=True)
        assert None == f.clean(None)
        assert {} == f.clean({})
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        assert {'foo': 'bar'} == f.clean({'foo': 'bar'})

    # ArrayField ##############################################################

    def test_arrayfield_1(self):
        f = ArrayField()
        pytest.raises(ValidationError, f.clean, None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        assert [] == f.clean([])
        assert ['foo', 'bar'], f.clean(['foo' == 'bar'])
        assert ['foo', 'bar', '$', 'eek'], f.clean(['foo', 'bar', '$' == 'eek'])
        assert f.default == list

    def test_arrayfield_2(self):
        f = ArrayField(null=True)
        assert None == f.clean(None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        assert [] == f.clean([])
        assert ['foo', 'bar'], f.clean(['foo' == 'bar'])
        assert ['foo', 'bar', '$', 'eek'], f.clean(['foo', 'bar', '$' == 'eek'])

    # TypedListField #########################################################

    def test_typedlistfield_1(self):
        f = TypedListField(IntegerField())
        assert "List<Integer>" == f.data_type_name(f)
        pytest.raises(ValidationError, f.clean, None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        assert [] == f.clean([])
        pytest.raises(ValidationError, f.clean, ['foo', 'bar'])
        assert [1, 2, 3], f.clean([1, 2 == 3])
        assert f.default == list

    def test_typedlistfield_2(self):
        f = TypedListField(IntegerField(), null=True)
        assert "List<Integer>" == f.data_type_name(f)
        assert None == f.clean(None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        assert [] == f.clean([])
        pytest.raises(ValidationError, f.clean, ['foo', 'bar'])
        assert [1, 2, 3], f.clean([1, 2 == 3])

    def test_typed_list_field_dynamic_type_name(self):
        f = TypedListField(TestDynamicTypeNameField(), null=True)
        assert "List<Foo>" == f.data_type_name(f)

    # TypedDictField ##########################################################

    def test_typeddictfield_1(self):
        f = TypedDictField(IntegerField())
        assert "Dict<String, Integer>" == f.data_type_name(f)
        pytest.raises(ValidationError, f.clean, None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        assert {} == f.clean({})
        pytest.raises(ValidationError, f.clean, {'foo': 'bar'})
        assert {'foo': 1} == f.clean({'foo': 1})

    def test_typeddictfield_2(self):
        f = TypedDictField(IntegerField(), null=True)
        assert "Dict<String, Integer>" == f.data_type_name(f)
        assert None == f.clean(None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        assert {} == f.clean({})
        pytest.raises(ValidationError, f.clean, {'foo': 'bar'})
        assert {'foo': 1} == f.clean({'foo': 1})

    def test_typeddictfield_3(self):
        f = TypedDictField(StringField(), IntegerField(), null=True)
        assert "Dict<Integer, String>" == f.data_type_name(f)
        pytest.raises(ValidationError, f.clean, {'foo': 'bar'})
        assert {1: 'foo'} == f.clean({1: 'foo'})

    def test_typeddictfield_nested_typed_array(self):
        f = TypedDictField(TypedArrayField(StringField()))
        assert "Dict<String, List<String>>" == f.data_type_name(f)
        assert {} == f.clean({})
        pytest.raises(ValidationError, f.clean, {'foo': 'bar'})
        assert {'foo': ['bar', 'eek']}, f.clean({'foo': ['bar' == 'eek']})

    def test_typeddictfield_validate(self):
        f = TypedDictField(
            IntegerField(min_value=5),
            StringField(max_length=5, choices=[
                ('foo', 'Foo'),
                ('bad_value', 'Bad Value'),
            ])
        )
        assert "Dict<String, Integer>" == f.data_type_name(f)
        pytest.raises(ValidationError, f.clean, {None: 6})
        pytest.raises(ValidationError, f.clean, {'bad_value': 6})
        pytest.raises(ValidationError, f.clean, {'bar': 6})
        pytest.raises(ValidationError, f.clean, {'foo': None})
        pytest.raises(ValidationError, f.clean, {'foo': 2})

    def test_typed_dict_field_dynamic_type_name(self):
        f = TypedDictField(
            TestDynamicTypeNameField(),
            TestDynamicTypeNameField(),
        )
        assert "Dict<Foo, Foo>" == f.data_type_name(f)




# -*- coding: utf-8 -*-
from __future__ import division
import decimal
import six

__all__ = ('Currency', 'set_default_currency', 'Amount')


class Currency(object):
    """
    A monetary currency.
    """
    def __new__(cls, code, number, name="", symbol=u"", precision=2):
        self = object.__new__(cls)
        self.code = code
        self.number = number
        self.name = name
        self.symbol = symbol
        self.precision = precision
        return self

    def __eq__(self, other):
        if isinstance(other, Currency):
            return self.code == other.code
        if isinstance(other, str):
            return self.code == other
        return False

    def __repr__(self):
        return "<Currency: %s>" % self.code

    def __str__(self):
        if self.name:
            return "%s - %s" % (self.name, self.code)
        else:
            return self.code


CURRENCY = {}
# XXX is the currency code for no currency in ISO 4217
CURRENCY['XXX'] = DEFAULT_CURRENCY = NO_CURRENCY = Currency('XXX', 999, 'No Currency', '', 4)


def set_default_currency(code='XXX'):
    """
    Set the default currency (the one applied if no currency is specified)
    :param code:
    :return:
    """
    global DEFAULT_CURRENCY
    DEFAULT_CURRENCY = CURRENCY[code]


class Amount(tuple):
    """
    A monetary amount and the associated currency.
    """
    def __new__(cls, value=None, currency=None):
        if isinstance(value, (tuple, list)):
            if len(value) == 1:
                value = value[0]
            elif len(value) == 2:
                value, currency = value
            else:
                raise ValueError("expected tuple with a length of 1 or 2 got: %s" % len(value))

        if not isinstance(value, decimal.Decimal):
            try:
                value = decimal.Decimal(value)
            except decimal.InvalidOperation:
                raise ValueError("value is not a valid numerical amount: %s" % value)

        if currency is None:
            currency = DEFAULT_CURRENCY
        elif isinstance(currency, six.string_types):
            currency = CURRENCY[currency.upper()]
        elif not isinstance(currency, Currency):
            raise TypeError("unable to convert value into a valid currency: %s" % currency)

        return tuple.__new__(cls, (value, currency))

    # Type conversions

    def __float__(self):
        return float(self.value)

    def __int__(self):
        return int(self.value)

    def __str__(self):
        if self.currency == NO_CURRENCY:
            return self.format("{value}")
        else:
            return self.format("{value} {currency.code}")

    def __repr__(self):
        return self.format("<Amount: {value}, {currency!r}>")

    # Math operations

    def __round__(self, n=None):
        """
        Round the value to the specified.
        """
        return self.value.__round__(n or self.currency.precision)

    def __pos__(self):
        return Amount(self.value, self.currency)

    def __neg__(self):
        return Amount(-self.value, self.currency)

    def __add__(self, other):
        other_value, currency = self._pre_calculate(other)
        return Amount(self.value + other_value, currency)

    def __sub__(self, other):
        other_value, currency = self._pre_calculate(other)
        return Amount(self.value - other_value, currency)

    def __mul__(self, other):
        if isinstance(other, Amount):
            raise TypeError("monetary amounts can not be multiplied.")
        else:
            return Amount(self.value * decimal.Decimal(other), self.currency)

    def __truediv__(self, other):
        other_value, currency = self._pre_calculate(other)
        # If the other instance is a currency we return just a value.
        if isinstance(other, Amount):
            return self.value / other_value
        else:
            return Amount(self.value / other_value, currency)

    __radd__ = __add__
    __rsub__ = __sub__
    __rmul__ = __mul__
    __div__ = __rtruediv__ = __truediv__

    # Comparison operators

    def __eq__(self, other):
        if isinstance(other, Amount):
            return self.value == other.value and self.currency == other.currency
        return False

    def __ne__(self, other):
        return not self.__eq__(other)

    def __lt__(self, other):
        other_value, _ = self._pre_calculate(other)
        return self.value < other_value

    def __gt__(self, other):
        other_value, _ = self._pre_calculate(other)
        return self.value > other_value

    def __le__(self, other):
        return self < other or self == other

    def __ge__(self, other):
        return self > other or self == other

    def _pre_calculate(self, other):
        """
        Handle any pre-calculation checks
        """
        if isinstance(other, Amount):
            other_value = other.value
            other_currency = other.currency
        else:
            other_value = decimal.Decimal(other)
            other_currency = NO_CURRENCY

        if self.currency == NO_CURRENCY:
            currency = other_currency
        elif other_currency == NO_CURRENCY:
            currency = self.currency
        elif self.currency != other.currency:
            raise ValueError("Cannot perform calculation on amounts of different currencies.")
        else:
            currency = self.currency

        return other_value, currency

    @property
    def value(self):
        return self[0]

    @property
    def currency(self):
        return self[1]

    @property
    def is_naive(self):
        """
        Boolean indicating if this field is assigned `NO_CURRENCY`.
        """
        return self.currency == NO_CURRENCY

    def format(self, format_string):
        """
        Format an amount.

        Internally this uses the python builtin string format method. The following fields are available for formatting:

        - *value*; value of the amount formatted to the precision defined by the currency
        - *value_raw*; the raw value used internally (a Decimal)
        - *currency*; the currency
          - *currency.symbol*; the symbol defined by the currency
          - *currency.code*; the code defined by the currency
          - *currency.number*; the number defined by the currency (currencies are assigned a number along with a code)
          - *currency.name*; the name defined by the currency

        Usage::

            > Amount(10, 'USD').format("{symbol}{value} {code}")
            $10.00 USD

        :param format_string:
        :return: formatted string

        """
        assert isinstance(format_string, six.string_types)
        return format_string.format(
            value=('{0:0.%if}' % self.currency.precision).format(self.value),
            value_raw=self.value,
            currency=self.currency,
        )

    def assign_currency(self, currency):
        """
        Assign a currency if this amount currently has no currency assigned.

        As an ``Amount`` is immutable this will return a new ``Amount`` instance.

        :param currency: Currency to apply.
        :return:

        """
        if not self.is_naive:
            raise ValueError("This amount is already assigned a currency.")
        return Amount(self.value, CURRENCY[currency])


# Currencies based off the latest ISO 4217 list of currencies
# Updated: 2014-08-29
CURRENCY['AED'] = Currency('AED', 784, 'UAE Dirham', u'', 2)
CURRENCY['AFN'] = Currency('AFN', 971, 'Afghani', u'؋', 2)
CURRENCY['ALL'] = Currency('ALL', 8, 'Lek', u'', 2)
CURRENCY['AMD'] = Currency('AMD', 51, 'Armenian Dram', u'', 2)
CURRENCY['ANG'] = Currency('ANG', 532, 'Netherlands Antillean Guilder', u'ƒ', 2)
CURRENCY['AOA'] = Currency('AOA', 973, 'Kwanza', u'', 2)
CURRENCY['ARS'] = Currency('ARS', 32, 'Argentine Peso', u'$', 2)
CURRENCY['AUD'] = Currency('AUD', 36, 'Australian Dollar', u'$', 2)
CURRENCY['AWG'] = Currency('AWG', 533, 'Aruban Florin', u'ƒ', 2)
CURRENCY['AZN'] = Currency('AZN', 944, 'Azerbaijanian Manat', u'ман', 2)
CURRENCY['BAM'] = Currency('BAM', 977, 'Convertible Mark', u'KM', 2)
CURRENCY['BBD'] = Currency('BBD', 52, 'Barbados Dollar', u'$', 2)
CURRENCY['BDT'] = Currency('BDT', 50, 'Taka', u'', 2)
CURRENCY['BGN'] = Currency('BGN', 975, 'Bulgarian Lev', u'', 2)
CURRENCY['BHD'] = Currency('BHD', 48, 'Bahraini Dinar', u'', 3)
CURRENCY['BIF'] = Currency('BIF', 108, 'Burundi Franc', u'', 0)
CURRENCY['BMD'] = Currency('BMD', 60, 'Bermudian Dollar', u'', 2)
CURRENCY['BND'] = Currency('BND', 96, 'Brunei Dollar', u'$', 2)
CURRENCY['BOB'] = Currency('BOB', 68, 'Boliviano', u'', 2)
CURRENCY['BOV'] = Currency('BOV', 984, 'Mvdol', u'', 2)
CURRENCY['BRL'] = Currency('BRL', 986, 'Brazilian Real', u'', 2)
CURRENCY['BSD'] = Currency('BSD', 44, 'Bahamian Dollar', u'', 2)
CURRENCY['BTN'] = Currency('BTN', 64, 'Ngultrum', u'', 2)
CURRENCY['BWP'] = Currency('BWP', 72, 'Pula', u'P', 2)
CURRENCY['BYR'] = Currency('BYR', 974, 'Belarussian Ruble', u'p.', 0)
CURRENCY['BZD'] = Currency('BZD', 84, 'Belize Dollar', u'BZ$', 2)
CURRENCY['CAD'] = Currency('CAD', 124, 'Canadian Dollar', u'$', 2)
CURRENCY['CDF'] = Currency('CDF', 976, 'Congolese Franc', u'', 2)
CURRENCY['CHE'] = Currency('CHE', 947, 'WIR Euro', u'', 2)
CURRENCY['CHF'] = Currency('CHF', 756, 'Swiss Franc', u'CHF', 2)
CURRENCY['CHW'] = Currency('CHW', 948, 'WIR Franc', u'', 2)
CURRENCY['CLF'] = Currency('CLF', 990, 'Unidad de Fomento', u'', 4)
CURRENCY['CLP'] = Currency('CLP', 152, 'Chilean Peso', u'$', 0)
CURRENCY['CNY'] = Currency('CNY', 156, 'Yuan Renminbi', u'¥', 2)
CURRENCY['COP'] = Currency('COP', 170, 'Colombian Peso', u'$', 2)
CURRENCY['COU'] = Currency('COU', 970, 'Unidad de Valor Real', u'', 2)
CURRENCY['CRC'] = Currency('CRC', 188, 'Costa Rican Colon', u'₡', 2)
CURRENCY['CUC'] = Currency('CUC', 931, 'Peso Convertible', u'', 2)
CURRENCY['CUP'] = Currency('CUP', 192, 'Cuban Peso', u'₱', 2)
CURRENCY['CVE'] = Currency('CVE', 132, 'Cabo Verde Escudo', u'', 2)
CURRENCY['CZK'] = Currency('CZK', 203, 'Czech Koruna', u'Kč', 2)
CURRENCY['DJF'] = Currency('DJF', 262, 'Djibouti Franc', u'', 0)
CURRENCY['DKK'] = Currency('DKK', 208, 'Danish Krone', u'kr', 2)
CURRENCY['DOP'] = Currency('DOP', 214, 'Dominican Peso', u'RD$', 2)
CURRENCY['DZD'] = Currency('DZD', 12, 'Algerian Dinar', u'', 2)
CURRENCY['EGP'] = Currency('EGP', 818, 'Egyptian Pound', u'£', 2)
CURRENCY['ERN'] = Currency('ERN', 232, 'Nakfa', u'', 2)
CURRENCY['ETB'] = Currency('ETB', 230, 'Ethiopian Birr', u'', 2)
CURRENCY['EUR'] = Currency('EUR', 978, 'Euro', u'€', 2)
CURRENCY['FJD'] = Currency('FJD', 242, 'Fiji Dollar', u'$', 2)
CURRENCY['FKP'] = Currency('FKP', 238, 'Falkland Islands Pound', u'£', 2)
CURRENCY['GBP'] = Currency('GBP', 826, 'Pound Sterling', u'£', 2)
CURRENCY['GEL'] = Currency('GEL', 981, 'Lari', u'', 2)
CURRENCY['GHS'] = Currency('GHS', 936, 'Ghana Cedi', u'¢', 2)
CURRENCY['GIP'] = Currency('GIP', 292, 'Gibraltar Pound', u'£', 2)
CURRENCY['GMD'] = Currency('GMD', 270, 'Dalasi', u'', 2)
CURRENCY['GNF'] = Currency('GNF', 324, 'Guinea Franc', u'', 0)
CURRENCY['GTQ'] = Currency('GTQ', 320, 'Quetzal', u'Q', 2)
CURRENCY['GYD'] = Currency('GYD', 328, 'Guyana Dollar', u'$', 2)
CURRENCY['HKD'] = Currency('HKD', 344, 'Hong Kong Dollar', u'$', 2)
CURRENCY['HNL'] = Currency('HNL', 340, 'Lempira', u'L', 2)
CURRENCY['HRK'] = Currency('HRK', 191, 'Croatian Kuna', u'kn', 2)
CURRENCY['HTG'] = Currency('HTG', 332, 'Gourde', u'', 2)
CURRENCY['HUF'] = Currency('HUF', 348, 'Forint', u'Ft', 2)
CURRENCY['IDR'] = Currency('IDR', 360, 'Rupiah', u'Rp', 2)
CURRENCY['ILS'] = Currency('ILS', 376, 'New Israeli Sheqel', u'₪', 2)
CURRENCY['INR'] = Currency('INR', 356, 'Indian Rupee', u'₹', 2)
CURRENCY['IQD'] = Currency('IQD', 368, 'Iraqi Dinar', u'', 3)
CURRENCY['IRR'] = Currency('IRR', 364, 'Iranian Rial', u'﷼', 2)
CURRENCY['ISK'] = Currency('ISK', 352, 'Iceland Krona', u'kr', 0)
CURRENCY['JMD'] = Currency('JMD', 388, 'Jamaican Dollar', u'J$', 2)
CURRENCY['JOD'] = Currency('JOD', 400, 'Jordanian Dinar', u'', 3)
CURRENCY['JPY'] = Currency('JPY', 392, 'Yen', u'¥', 0)
CURRENCY['KES'] = Currency('KES', 404, 'Kenyan Shilling', u'', 2)
CURRENCY['KGS'] = Currency('KGS', 417, 'Som', u'лв', 2)
CURRENCY['KHR'] = Currency('KHR', 116, 'Riel', u'៛', 2)
CURRENCY['KMF'] = Currency('KMF', 174, 'Comoro Franc', u'', 0)
CURRENCY['KPW'] = Currency('KPW', 408, 'North Korean Won', u'₩', 2)
CURRENCY['KRW'] = Currency('KRW', 410, 'Won', u'₩', 0)
CURRENCY['KWD'] = Currency('KWD', 414, 'Kuwaiti Dinar', u'', 3)
CURRENCY['KYD'] = Currency('KYD', 136, 'Cayman Islands Dollar', u'$', 2)
CURRENCY['KZT'] = Currency('KZT', 398, 'Tenge', u'лв', 2)
CURRENCY['LAK'] = Currency('LAK', 418, 'Kip', u'₭', 2)
CURRENCY['LBP'] = Currency('LBP', 422, 'Lebanese Pound', u'£', 2)
CURRENCY['LKR'] = Currency('LKR', 144, 'Sri Lanka Rupee', u'₨', 2)
CURRENCY['LRD'] = Currency('LRD', 430, 'Liberian Dollar', u'$', 2)
CURRENCY['LSL'] = Currency('LSL', 426, 'Loti', u'', 2)
CURRENCY['LTL'] = Currency('LTL', 440, 'Lithuanian Litas', u'Lt', 2)
CURRENCY['LYD'] = Currency('LYD', 434, 'Libyan Dinar', u'', 3)
CURRENCY['MAD'] = Currency('MAD', 504, 'Moroccan Dirham', u'', 2)
CURRENCY['MDL'] = Currency('MDL', 498, 'Moldovan Leu', u'', 2)
CURRENCY['MGA'] = Currency('MGA', 969, 'Malagasy Ariary', u'', 2)
CURRENCY['MKD'] = Currency('MKD', 807, 'Denar', u'ден', 2)
CURRENCY['MMK'] = Currency('MMK', 104, 'Kyat', u'', 2)
CURRENCY['MNT'] = Currency('MNT', 496, 'Tugrik', u'₮', 2)
CURRENCY['MOP'] = Currency('MOP', 446, 'Pataca', u'', 2)
CURRENCY['MRO'] = Currency('MRO', 478, 'Ouguiya', u'', 2)
CURRENCY['MUR'] = Currency('MUR', 480, 'Mauritius Rupee', u'₨', 2)
CURRENCY['MVR'] = Currency('MVR', 462, 'Rufiyaa', u'', 2)
CURRENCY['MWK'] = Currency('MWK', 454, 'Kwacha', u'', 2)
CURRENCY['MXN'] = Currency('MXN', 484, 'Mexican Peso', u'$', 2)
CURRENCY['MXV'] = Currency('MXV', 979, 'Mexican Unidad de Inversion (UDI)', u'', 2)
CURRENCY['MYR'] = Currency('MYR', 458, 'Malaysian Ringgit', u'RM', 2)
CURRENCY['MZN'] = Currency('MZN', 943, 'Mozambique Metical', u'MT', 2)
CURRENCY['NAD'] = Currency('NAD', 516, 'Namibia Dollar', u'$', 2)
CURRENCY['NGN'] = Currency('NGN', 566, 'Naira', u'₦', 2)
CURRENCY['NIO'] = Currency('NIO', 558, 'Cordoba Oro', u'C$', 2)
CURRENCY['NOK'] = Currency('NOK', 578, 'Norwegian Krone', u'kr', 2)
CURRENCY['NPR'] = Currency('NPR', 524, 'Nepalese Rupee', u'₨', 2)
CURRENCY['NZD'] = Currency('NZD', 554, 'New Zealand Dollar', u'$', 2)
CURRENCY['OMR'] = Currency('OMR', 512, 'Rial Omani', u'﷼', 3)
CURRENCY['PAB'] = Currency('PAB', 590, 'Balboa', u'B/.', 2)
CURRENCY['PEN'] = Currency('PEN', 604, 'Nuevo Sol', u'S/.', 2)
CURRENCY['PGK'] = Currency('PGK', 598, 'Kina', u'', 2)
CURRENCY['PHP'] = Currency('PHP', 608, 'Philippine Peso', u'₱', 2)
CURRENCY['PKR'] = Currency('PKR', 586, 'Pakistan Rupee', u'₨', 2)
CURRENCY['PLN'] = Currency('PLN', 985, 'Zloty', u'zł', 2)
CURRENCY['PYG'] = Currency('PYG', 600, 'Guarani', u'Gs', 0)
CURRENCY['QAR'] = Currency('QAR', 634, 'Qatari Rial', u'﷼', 2)
CURRENCY['RON'] = Currency('RON', 946, 'New Romanian Leu', u'lei', 2)
CURRENCY['RSD'] = Currency('RSD', 941, 'Serbian Dinar', u'Дин.', 2)
CURRENCY['RUB'] = Currency('RUB', 643, 'Russian Ruble', u'руб', 2)
CURRENCY['RWF'] = Currency('RWF', 646, 'Rwanda Franc', u'', 0)
CURRENCY['SAR'] = Currency('SAR', 682, 'Saudi Riyal', u'', 2)
CURRENCY['SBD'] = Currency('SBD', 90, 'Solomon Islands Dollar', u'$', 2)
CURRENCY['SCR'] = Currency('SCR', 690, 'Seychelles Rupee', u'₨', 2)
CURRENCY['SDG'] = Currency('SDG', 938, 'Sudanese Pound', u'', 2)
CURRENCY['SEK'] = Currency('SEK', 752, 'Swedish Krona', u'kr', 2)
CURRENCY['SGD'] = Currency('SGD', 702, 'Singapore Dollar', u'$', 2)
CURRENCY['SHP'] = Currency('SHP', 654, 'Saint Helena Pound', u'£', 2)
CURRENCY['SLL'] = Currency('SLL', 694, 'Leone', u'', 2)
CURRENCY['SOS'] = Currency('SOS', 706, 'Somali Shilling', u'S', 2)
CURRENCY['SRD'] = Currency('SRD', 968, 'Surinam Dollar', u'$', 2)
CURRENCY['SSP'] = Currency('SSP', 728, 'South Sudanese Pound', u'', 2)
CURRENCY['STD'] = Currency('STD', 678, 'Dobra', u'', 2)
CURRENCY['SVC'] = Currency('SVC', 222, 'El Salvador Colon', u'$', 2)
CURRENCY['SYP'] = Currency('SYP', 760, 'Syrian Pound', u'£', 2)
CURRENCY['SZL'] = Currency('SZL', 748, 'Lilangeni', u'', 2)
CURRENCY['THB'] = Currency('THB', 764, 'Baht', u'฿', 2)
CURRENCY['TJS'] = Currency('TJS', 972, 'Somoni', u'', 2)
CURRENCY['TMT'] = Currency('TMT', 934, 'Turkmenistan New Manat', u'', 2)
CURRENCY['TND'] = Currency('TND', 788, 'Tunisian Dinar', u'', 3)
CURRENCY['TOP'] = Currency('TOP', 776, 'Pa’anga', u'', 2)
CURRENCY['TRY'] = Currency('TRY', 949, 'Turkish Lira', u'₤', 2)
CURRENCY['TTD'] = Currency('TTD', 780, 'Trinidad and Tobago Dollar', u'TT$', 2)
CURRENCY['TWD'] = Currency('TWD', 901, 'New Taiwan Dollar', u'NT$', 2)
CURRENCY['TZS'] = Currency('TZS', 834, 'Tanzanian Shilling', u'', 2)
CURRENCY['UAH'] = Currency('UAH', 980, 'Hryvnia', u'₴', 2)
CURRENCY['UGX'] = Currency('UGX', 800, 'Uganda Shilling', u'', 0)
CURRENCY['USD'] = Currency('USD', 840, 'US Dollar', u'$', 2)
CURRENCY['USN'] = Currency('USN', 997, 'US Dollar (Next day)', u'', 2)
CURRENCY['UYI'] = Currency('UYI', 940, 'Uruguay Peso en Unidades Indexadas (URUIURUI)', u'', 0)
CURRENCY['UYU'] = Currency('UYU', 858, 'Peso Uruguayo', u'$U', 2)
CURRENCY['UZS'] = Currency('UZS', 860, 'Uzbekistan Sum', u'лв', 2)
CURRENCY['VEF'] = Currency('VEF', 937, 'Bolivar', u'Bs', 2)
CURRENCY['VND'] = Currency('VND', 704, 'Dong', u'₫', 0)
CURRENCY['VUV'] = Currency('VUV', 548, 'Vatu', u'', 0)
CURRENCY['WST'] = Currency('WST', 882, 'Tala', u'', 2)
CURRENCY['XAF'] = Currency('XAF', 950, 'CFA Franc BEAC', u'', 0)
CURRENCY['XCD'] = Currency('XCD', 951, 'East Caribbean Dollar', u'$', 2)
CURRENCY['XOF'] = Currency('XOF', 952, 'CFA Franc BCEAO', u'', 0)
CURRENCY['XPF'] = Currency('XPF', 953, 'CFP Franc', u'', 0)
CURRENCY['XTS'] = Currency('XTS', 963, 'Test currency', u'', 0)
CURRENCY['YER'] = Currency('YER', 886, 'Yemeni Rial', u'﷼', 2)
CURRENCY['ZAR'] = Currency('ZAR', 710, 'Rand', u'S', 2)
CURRENCY['ZMW'] = Currency('ZMW', 967, 'Zambian Kwacha', u'', 2)
CURRENCY['ZWL'] = Currency('ZWL', 932, 'Zimbabwe Dollar', u'Z$', 2)

# Additional special case as BTC is not a "real" currency.
CURRENCY['BTC'] = Currency('BTC', 0, 'Bitcoin', u'฿', 4)



# -*- coding: utf-8 -*-
import pytest
import odin
from odin.exceptions import ValidationError
from odin.fields.composite import DictAs, ArrayOf, DictOf


class ExampleResource(odin.Resource):
    name = odin.StringField()


class TestFields(object):
    def assertResourceEqual(self, first, second):
        if not first:
            raise AssertionError("First item is None")
        if not second:
            raise AssertionError("Second item is None")

        if first._meta.resource_name != second._meta.resource_name:
            raise AssertionError("Resources are different types %s != %s" % (
                first._meta.resource_name, second._meta.resource_name))

        for f in first._meta.fields:
            first_val = getattr(first, f.name)
            second_val = getattr(second, f.name)
            if first_val != second_val:
                raise AssertionError("Resource field %s does not match '%s' != '%s'" % (f.name, first_val, second_val))

    def assertResourceListEqual(self, first, second):
        if not isinstance(first, list):
            raise AssertionError("First is not a list")
        if not isinstance(second, list):
            raise AssertionError("Second is not a list")
        if len(first) != len(second):
            raise AssertionError("Lists are different lengths")
        for idx in range(len(first)):
            self.assertResourceEqual(first[idx], second[idx])

    def assertResourceDictEqual(self, first, second):
        if not isinstance(first, dict):
            raise AssertionError("First is not a dict")
        if not isinstance(second, dict):
            raise AssertionError("Second is dict")
        if len(first) != len(second):
            raise AssertionError("Dicts are different lengths")
        for key in first.keys():
            self.assertResourceEqual(first[key], second[key])

    # DictAs ##################################################################

    def test_dictas_ensure_is_resource(self):
        with pytest.raises(TypeError):
            DictAs("an item")

    def test_dictas_1(self):
        f = DictAs(ExampleResource)
        pytest.raises(ValidationError, f.clean, None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        pytest.raises(ValidationError, f.clean, {})
        pytest.raises(ValidationError, f.clean, {'$': 'tests.test_composite_fields.ExampleResource'})
        self.assertResourceEqual(ExampleResource(name='foo'), f.clean({'name': 'foo'}))
        self.assertResourceEqual(ExampleResource(name='foo'), f.clean({
            '$': 'tests.test_composite_fields.ExampleResource', 'name': 'foo'}))

    def test_dictas_2(self):
        f = DictAs(ExampleResource, null=True)
        assert f.clean(None) is None
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        pytest.raises(ValidationError, f.clean, {})
        pytest.raises(ValidationError, f.clean, {'$': 'tests.test_composite_fields.ExampleResource'})
        self.assertResourceEqual(ExampleResource(name='foo'), f.clean({'name': 'foo'}))
        self.assertResourceEqual(ExampleResource(name='foo'), f.clean({
            '$': 'tests.test_composite_fields.ExampleResource', 'name': 'foo'}))

    # ArrayOf #################################################################

    def test_arrayof_1(self):
        f = ArrayOf(ExampleResource)
        pytest.raises(ValidationError, f.clean, None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        pytest.raises(ValidationError, f.clean, {})
        pytest.raises(ValidationError, f.clean, {'$': 'tests.test_composite_fields.ExampleResource'})
        pytest.raises(ValidationError, f.clean, [None])
        self.assertResourceListEqual([], f.clean([]))
        self.assertResourceListEqual([ExampleResource(name='foo')], f.clean([{'name': 'foo'}]))
        self.assertResourceListEqual([ExampleResource(name='foo')], f.clean([{
            '$': 'tests.test_composite_fields.ExampleResource', 'name': 'foo'}]))

    def test_arrayof_2(self):
        f = ArrayOf(ExampleResource, null=True)
        assert f.clean(None) is None
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        pytest.raises(ValidationError, f.clean, {})
        pytest.raises(ValidationError, f.clean, {'$': 'tests.test_composite_fields.ExampleResource'})
        pytest.raises(ValidationError, f.clean, [None])
        self.assertResourceListEqual([], f.clean([]))
        self.assertResourceListEqual([ExampleResource(name='foo')], f.clean([{'name': 'foo'}]))
        self.assertResourceListEqual([ExampleResource(name='foo')], f.clean([{
            '$': 'tests.test_composite_fields.ExampleResource', 'name': 'foo'}]))

    def test_arrayof_3(self):
        f = ArrayOf(ExampleResource, empty=False)
        pytest.raises(ValidationError, f.clean, None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        pytest.raises(ValidationError, f.clean, {})
        pytest.raises(ValidationError, f.clean, {'$': 'tests.test_composite_fields.ExampleResource'})
        pytest.raises(ValidationError, f.clean, [None])
        pytest.raises(ValidationError, f.clean, [])
        self.assertResourceListEqual([ExampleResource(name='foo')], f.clean([{'name': 'foo'}]))
        self.assertResourceListEqual([ExampleResource(name='foo')], f.clean([{
            '$': 'tests.test_composite_fields.ExampleResource', 'name': 'foo'}]))

    # DictOf #################################################################

    def test_dictof_1(self):
        f = DictOf(ExampleResource)
        pytest.raises(ValidationError, f.clean, None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        pytest.raises(ValidationError, f.clean, [])
        pytest.raises(ValidationError, f.clean, {'$': 'tests.test_composite_fields.ExampleResource'})
        pytest.raises(ValidationError, f.clean, {'abc': None})
        self.assertResourceDictEqual({}, f.clean({}))
        self.assertResourceDictEqual({'foo': ExampleResource(name='foo')}, f.clean({'foo': {'name': 'foo'}}))
        self.assertResourceDictEqual({'foo': ExampleResource(name='foo')}, f.clean({'foo': {
            '$': 'tests.test_composite_fields.ExampleResource', 'name': 'foo'}}))

    def test_dictof_2(self):
        f = DictOf(ExampleResource, null=True)
        assert f.clean(None) is None
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        pytest.raises(ValidationError, f.clean, [])
        pytest.raises(ValidationError, f.clean, {'$': 'tests.test_composite_fields.ExampleResource'})
        pytest.raises(ValidationError, f.clean, {'abc': None})
        self.assertResourceDictEqual({}, f.clean({}))
        self.assertResourceDictEqual({'foo': ExampleResource(name='foo')}, f.clean({'foo': {'name': 'foo'}}))
        self.assertResourceDictEqual({'foo': ExampleResource(name='foo')}, f.clean({'foo': {
            '$': 'tests.test_composite_fields.ExampleResource', 'name': 'foo'}}))

    def test_dictof_3(self):
        f = DictOf(ExampleResource, empty=False)
        pytest.raises(ValidationError, f.clean, None)
        pytest.raises(ValidationError, f.clean, 'abc')
        pytest.raises(ValidationError, f.clean, 123)
        pytest.raises(ValidationError, f.clean, [])
        pytest.raises(ValidationError, f.clean, {'$': 'tests.test_composite_fields.ExampleResource'})
        pytest.raises(ValidationError, f.clean, {'abc': None})
        pytest.raises(ValidationError, f.clean, {})
        self.assertResourceDictEqual({'foo': ExampleResource(name='foo')}, f.clean({'foo': {'name': 'foo'}}))
        self.assertResourceDictEqual({'foo': ExampleResource(name='foo')}, f.clean({'foo': {
            '$': 'tests.test_composite_fields.ExampleResource', 'name': 'foo'}}))

    def test_dictof_key_choices(self):
        f = DictOf(ExampleResource, null=True, key_choices=(
            ('foo', 'Foo'),
            (None, 'None'),
        ))
        pytest.raises(ValidationError, f.clean, {'eek': ExampleResource(name='foo')})
        self.assertResourceDictEqual({'foo': ExampleResource(name='foo')}, f.clean({'foo': {'name': 'foo'}}))



import json
import pytest
import six


def assertJSONEqual(raw, expected_data, msg=None):
    """
    Assert that two JSON documents are the same.
    """
    try:
        data = json.loads(raw)
    except ValueError:
        pytest.fail("First argument is not valid JSON: %r" % raw)
    else:
        if isinstance(expected_data, six.string_types):
            try:
                expected_data = json.loads(expected_data)
            except ValueError:
                pytest.fail("Second argument is not valid JSON: %r" % expected_data)

        if data != expected_data:
            pytest.fail(msg)



# -*- coding: utf-8 -*-
import re

_CAMEL_CASE_RE = re.compile(r'[A-Z]')
_LOWER_UNDERSCORE_CASE_RE = re.compile(r'_([a-z])')
_LOWER_DASH_CASE_RE = re.compile(r'-([a-z])')


def camel_to_lower_separated(s, sep):
    """
    Convert camel case representation into lower separated case ie:

      backgroundColor -> background_color

    Note any separator at the start or end is stripped.

    """
    return _CAMEL_CASE_RE.sub(lambda m: sep + m.group(0).lower(), s).strip(sep)


def camel_to_lower_underscore(s):
    """
    Convert camel case to lower underscore case.

        backgroundColor -> background_color
    """
    return camel_to_lower_separated(s, '_')


def camel_to_lower_dash(s):
    """
    Convert camel case to lower dash case.

      backgroundColor -> background-color
    """
    return camel_to_lower_separated(s, '-')


def lower_underscore_to_camel(value):
    """
    Convert lower underscore case to camel case

      background_color -> backgroundColor
    """
    return _LOWER_UNDERSCORE_CASE_RE.sub(
        lambda m: m.group(1).upper(),
        value.lower()
    )


def lower_dash_to_camel(value):
    """
    Convert lower dash case to camel case

      background-color -> backgroundColor
    """
    return _LOWER_DASH_CASE_RE.sub(
        lambda m: m.group(1).upper(),
        value.lower()
    )


class cached_property(object):
    """
    Acts like a standard class `property` except return values cached.
    """
    @staticmethod
    def clear_caches(instance):
        instance._cache = {}

    def __init__(self, func):
        self.func = func
        self.__doc__ = func.__doc__
        self.__name__ = func.__name__
        self.__module__ = func.__module__

    def __get__(self, instance, owner):
        try:
            value = instance._cache[self.__name__]
        except (KeyError, AttributeError):
            value = self.func(instance)
            try:
                cache = instance._cache
            except AttributeError:
                cache = instance._cache = {}
            cache[self.__name__] = value
        return value


def field_iter(resource, include_virtual=True):
    """
    Return an iterator that yields fields from a resource.

    :param resource: Resource to iterate over.
    :param include_virtual: Include virtual fields.
    :returns: an iterator that returns fields.

    """
    if include_virtual:
        return iter(resource._meta.all_fields)
    else:
        return iter(resource._meta.fields)


def field_iter_items(resource, fields=None):
    """
    Return an iterator that yields fields and their values from a resource.

    :param resource: Resource to iterate over.
    :param fields: Fields to use; if :const:`None` defaults to all of the resources fields.
    :returns: an iterator that returns (field, value) tuples.

    """
    if fields is None:
        fields = resource._meta.all_fields
    for f in fields:
        yield f, f.prepare(f.value_from_object(resource))


def virtual_field_iter_items(resource):
    """
    Return an iterator that yields virtual fields and their values from a resource.

    :param resource: Resource to iterate over.
    :returns: an iterator that returns (field, value) tuples.

    """
    return field_iter_items(resource, resource._meta.virtual_fields)


def attribute_field_iter_items(resource):
    """
    Return an iterator that yields fields and their values from a resource that have the attribute flag set.

    :param resource: Resource to iterate over.
    :returns: an iterator that returns (field, value) tuples.

    :note::
        This iterator is designed for codecs that have a distinction between attribute and element fields (eg XML).

    """
    return field_iter_items(resource, resource._meta.attribute_fields)


def element_field_iter_items(resource):
    """
    Return an iterator that yields fields and their values from a resource that do not have the attribute flag set.

    :param resource: Resource to iterate over.
    :returns: an iterator that returns (field, value) tuples.

    :note::
        This iterator is designed for codecs that have a distinction between attribute and element fields (eg XML).

    """
    return field_iter_items(resource, resource._meta.element_fields)


def extract_fields_from_dict(d, resource):
    """
    Extract values from a dict that are defined on a resource.

    Fields that are not found will not be included in the output dict.

    :param d: the source dictionary.
    :param resource: the resource that provides the fields.
    :returns: a dictionary of the resource fields that where found in the dict.

    """
    return dict((f.name, d[f.name]) for f in field_iter(resource) if f.name in d)


def value_in_choices(value, choices):
    """
    Check if the value appears in the choices list (a iterable of tuples, the first value of which is the choice value)

    :param value:
    :param choices:
    :return: True if value is in the choices iterable.

    """
    for choice in choices:
        if value == choice[0]:
            return True
    return False



# -*- coding: utf-8 -*-
import unittest
from odin import utils
from .resources import Book


class TestCamelAndDashFormats(object):
    def test_camel_to_lower_separated(self):
        assert "this_is_an_example" == utils.camel_to_lower_underscore('thisIsAnExample')
        assert "this_is_an_example" == utils.camel_to_lower_underscore('ThisIsAnExample')
        assert "this-is-a-test" == utils.camel_to_lower_dash('thisIsATest')

    def test_lower_underscore_to_camel(self):
        assert "thisIsAnExample" == utils.lower_underscore_to_camel('this_is_an_example')
        assert "thisIsAnExample" == utils.lower_underscore_to_camel('this_is_an_example')
        assert "thisIsATest" == utils.lower_underscore_to_camel('this_Is_A_test')

    def test_lower_dash_to_camel(self):
        assert "thisIsAnExample" == utils.lower_dash_to_camel('this-is-an-example')
        assert "thisIsAnExample" == utils.lower_dash_to_camel('this-is-an-example')
        assert "thisIsATest" == utils.lower_dash_to_camel('this-Is-A-test')


class TestIterator(object):
    def test_extract_fields_from_dict(self):
        source_dict = {
            'title': "My Title",
            'num_pages': 1,
            'rrp': 1.2,
            'fiction': True,
            'published': ['no_validation_for_type'],
            'another_field': 'that is not on the resource'
        }

        assert {
            'title': "My Title",
            'num_pages': 1,
            'rrp': 1.2,
            'fiction': True,
            'published': ['no_validation_for_type'],
        } == utils.extract_fields_from_dict(source_dict, Book)


class TestValueInChoice(object):
    CHOICES = (
        ('foo', 'Foo'),
        ('bar', 'Foo'),
        (123, 'Number'),
        (True, 'Boolean'),
        (None, 'None'),
        ('eek', 'With Help', 'Help string'),
    )

    def test_value_in_choices(self):
        assert utils.value_in_choices('foo', self.CHOICES)
        assert utils.value_in_choices(123, self.CHOICES)
        assert utils.value_in_choices(True, self.CHOICES)
        assert utils.value_in_choices(None, self.CHOICES)
        assert utils.value_in_choices('eek', self.CHOICES)
        assert not utils.value_in_choices('xyz', self.CHOICES)
        assert not utils.value_in_choices(False, self.CHOICES)
        assert not utils.value_in_choices(321, self.CHOICES)



from __future__ import absolute_import
import datetime
from odin.codecs import dict_codec
from odin.datetimeutil import utc
from .resources import *


class TestDictCodec(object):
    def test_dump(self):
        in_resource = Book(
            title='Consider Phlebas',
            isbn='0-333-45430-8',
            num_pages=471,
            rrp=19.50,
            fiction=True,
            genre="sci-fi",
            authors=[Author(name="Iain M. Banks")],
            publisher=Publisher(name="Macmillan"),
        )

        actual = dict_codec.dump(in_resource)
        assert actual == {
            '$': 'library.Book',
            'authors': [{
                '$': 'Author',
                'name': "Iain M. Banks",
            }],
            'fiction': True,
            'genre': "sci-fi",
            'num_pages': 471,
            'rrp': 19.50,
            'isbn': '0-333-45430-8',
            'title': 'Consider Phlebas',
            'published': [],
            'publisher': {
                '$': 'Publisher',
                'name': "Macmillan",
            },
        }

    def test_dump_no_type_field(self):
        in_resource = Book(
            title='Consider Phlebas',
            isbn='0-333-45430-8',
            num_pages=471,
            rrp=19.50,
            fiction=True,
            genre="sci-fi",
            authors=[Author(name="Iain M. Banks")],
            publisher=Publisher(name="Macmillan"),
        )

        actual = dict_codec.dump(in_resource, include_type_field=False)
        assert actual == {
            'authors': [{
                'name': "Iain M. Banks",
            }],
            'fiction': True,
            'genre': "sci-fi",
            'num_pages': 471,
            'rrp': 19.50,
            'isbn': '0-333-45430-8',
            'title': 'Consider Phlebas',
            'published': [],
            'publisher': {
                'name': "Macmillan",
            },
        }

    def test_dump_and_load_(self):
        in_resource = Book(
            title='Consider Phlebas',
            isbn='0-333-45430-8',
            num_pages=471,
            rrp=19.50,
            fiction=True,
            genre="sci-fi",
            authors=[Author(name="Iain M. Banks")],
            publisher=Publisher(name="Macmillan"),
            published=[datetime.datetime(1987, 1, 1, tzinfo=utc)]
        )

        d = dict_codec.dump(in_resource)
        out_resource = dict_codec.load(d)

        assert out_resource.title == in_resource.title
        assert out_resource.isbn == in_resource.isbn
        assert out_resource.num_pages == in_resource.num_pages
        assert out_resource.rrp == in_resource.rrp
        assert out_resource.fiction == in_resource.fiction
        assert out_resource.genre == in_resource.genre
        assert out_resource.authors[0].name == in_resource.authors[0].name
        assert out_resource.publisher.name == in_resource.publisher.name
        assert out_resource.published[0] == in_resource.published[0]



# -*- coding: utf-8 -*-
import os
import pytest
import odin
from odin.codecs.csv_codec import ResourceReader

FIXTURE_PATH_ROOT = os.path.join(os.path.dirname(__file__), "fixtures")


class Book(odin.Resource):
    title = odin.StringField(name="Title")
    num_pages = odin.IntegerField(name="Num Pages")
    rrp = odin.FloatField(name="RRP")
    genre = odin.StringField(name="Genre", choices=(
        ('sci-fi', 'Science Fiction'),
        ('fantasy', 'Fantasy'),
        ('others', 'Others'),
    ))
    author = odin.StringField(name="Author")
    publisher = odin.StringField(name="Publisher")


class TestCsvResourceReader(object):
    def test_valid(self):
        with open(os.path.join(FIXTURE_PATH_ROOT, "libary-valid.csv")) as f:
            books = [book for book in ResourceReader(f, Book)]

        assert 6 == len(books)
        assert "Consider Phlebas" == books[0].title

    def test_invalid(self):
        with open(os.path.join(FIXTURE_PATH_ROOT, "libary-invalid.csv")) as f:
            with pytest.raises(odin.exceptions.ValidationError):
                books = [book for book in ResourceReader(f, Book)]



# -*- coding: utf-8 -*-
"""
Some helpful decorators.
"""
from odin.resources import build_object_graph


def returns_resource(func=None, codec=None, resource=None, full_clean=True, codec_opts=None):
    """
    Apply to a function that returns data that can be converted into a resource (or resources).

    If a codec is provided the data will be converted using the codec, if not it will be assumed that the supplied data
    is a dictionary that can be converted into a resource.

    Note that this decorator can throw ``ValidationError`` exceptions.

    :param codec: Codec that should be used to convert data.
    :param resource: Resource to convert to (only required if the data does not contain a resource identifier).
    :param full_clean: Perform a full clean on the data post conversion.
    :param codec_opts: Options that should be supplied ot the codec (in the form of a dictionary).

    :return: Resource, Array of resources or None.
    """
    def outer(func):  # noqa
        def inner(*args, **kwargs):
            data = func(*args, **kwargs)
            if codec:
                opts = codec_opts or {}
                opts.setdefault('full_clean', full_clean)
                return codec.loads(data, resource, **opts)
            else:
                return build_object_graph(data, resource, full_clean)
        return inner

    return outer(func) if func else outer



# -*- coding: utf-8 -*-
from sphinx.ext.autodoc import Documenter, ModuleLevelDocumenter, bool_option
import odin
from odin.utils import field_iter


def reference_to(obj):
    if hasattr(obj, '_meta'):
        return ":py:class:`%s`" % obj._meta.name
    return obj


class ResourceDocumenter(ModuleLevelDocumenter):
    """
    Extension to ``sphinx.ext.autodoc`` to support documenting Odin resources.

    This extension supports output in two forms, class doc form (for documenting the python objects) and API form for
    documenting information about fields for use in an API document.

    Usage::

        .. autoodin-resource:: namespace.path.to.your.Resource
            :include_virtual:
            :hide_choices:

    To select API form use the *include_virtual* option.

    """
    objtype = 'odin-resource'
    directivetype = 'class'
    content_indent = ''

    option_spec = dict(
        include_virtual=bool_option,
        include_validators=bool_option,
        hide_choices=bool_option,
        **Documenter.option_spec
    )

    @classmethod
    def can_document_member(cls, member, membername, isattr, parent):
        return isinstance(member, odin.Resource)

    def add_directive_header(self, sig):
        self.add_line(".. py:currentmodule:: %s" % self.object._meta.name_space, '<odin_sphinx>')
        self.add_line(".. py:class:: %s" % self.object._meta.name, '<odin_sphinx>')
        if self.options.noindex:
            self.add_line('   :noindex:', '<odin_sphinx>')

    def build_field_triple(self, field):
        """
        Build a field triple of (name, data_type, details)

        :param field: Field to build triple from.
        :return: (name, data_type, details) of a field.

        """
        details = []
        if field.doc_text:
            details.append(field.doc_text)

        if field.has_default():
            details.append("\n\nDefault value: %s\n" % reference_to(field.get_default()))

        if not self.options.hide_choices and field.choices:
            details.append("\n\nChoices:\n")
            for value, label in field.choices:
                details.append("* %s - %s" % (value, label))

        if self.options.include_validators and field.validators:
            details.append("\n\nValidation rules:\n")
            for validator in field.validators:
                details.append("* %s" % validator)

        # Generate the name of the type this field represents
        type_name = field.data_type_name
        max_length = getattr(field, 'max_length', None)
        if max_length:
            type_name = "%s [%s]" % (type_name, max_length)
        if callable(type_name):
            type_name = type_name(field)
        if isinstance(field, odin.CompositeField):
            type_name = "%s %s" % (type_name, reference_to(field.of))

        return (
            "*%s*" % field.name if field.null else field.name,  # Name
            reference_to(type_name or "Unknown"),  # Data-type
            '\n'.join(details).split('\n')  # Details
        )

    def document_members(self, all_members=False):
        data_table = [
            self.build_field_triple(f) for f in field_iter(self.object, self.options.include_virtual)
        ]

        # Calculate table column widths
        name_len = 4
        data_type_len = 9
        details_len = 7
        for name, data_type, details in data_table:
            name_len = max(len(name), name_len)
            data_type_len = max(len(data_type), data_type_len)
            details_len = max(max(len(l) for l in details), details_len)
        name_len += 2  # Padding
        data_type_len += 2  # Padding
        details_len += 2  # Padding

        def add_separator(char='-'):
            self.add_line("+%s+%s+%s+" % (
                char * name_len,
                char * data_type_len,
                char * details_len
            ), '<odin_sphinx>')

        def add_row_line(name, data_type, details):
            self.add_line("| %s%s | %s%s | %s%s |" % (
                name, ' ' * (name_len - len(name) - 2),
                data_type, ' ' * (data_type_len - len(data_type) - 2),
                details, ' ' * (details_len - len(details) - 2),
            ), '<odin_sphinx>')

        def add_row(name, data_type, details):
            add_row_line(name, data_type, details.pop(0))
            for line in details:
                add_row_line('', '', line)

        # Generate table
        add_separator()
        add_row("Name", "Data type", ["Details"])
        add_separator('=')
        for row in data_table:
            add_row(*row)
            add_separator()


def setup(app):
    app.add_autodocumenter(ResourceDocumenter)



import pytest
import odin
from odin import traversal
from odin.exceptions import NoMatchError, InvalidPathError, MultipleMatchesError


class Level3(odin.Resource):
    class Meta:
        key_field_name = 'name'
        namespace = 'odin.traversal'

    name = odin.StringField()


class Level2(odin.Resource):
    class Meta:
        namespace = 'odin.traversal'

    name = odin.StringField()
    label = odin.StringField(null=True)
    level3s = odin.ListOf(Level3)


class Level1(odin.Resource):
    class Meta:
        namespace = 'odin.traversal'

    name = odin.StringField()
    level2 = odin.DictAs(Level2)
    level2s = odin.DictOf(Level2)


TEST_STRUCTURE = Level1(
    name='a',
    level2=Level2(name='b', level3s=[]),
    level2s=dict(
        a=Level2(name='c', level3s=[]),
        b=Level2(name='d', label='not_empty', level3s=[Level3(name='e'), Level3(name='f')]),
        c=Level2(name='g', label='not_empty', level3s=[Level3(name='h')]),
    )
)

TEST_LIST_STRUCTURE = [
    Level1(
        name='a',
        level2=Level2(name='b', level3s=[]),
        level2s=dict(
            a=Level2(name='c', level3s=[]),
            b=Level2(name='d', level3s=[Level3(name='e'), Level3(name='f')]),
            c=Level2(name='g', level3s=[Level3(name='h')]),
        )
    ),
    Level1(
        name='i',
        level2=Level2(name='j', level3s=[]),
        level2s=dict(
            a=Level2(name='k', level3s=[]),
            b=Level2(name='l', level3s=[Level3(name='m'), Level3(name='n')]),
            c=Level2(name='o', level3s=[Level3(name='p')]),
        )
    )
]


class TestResourceTraversalIterator(traversal.ResourceTraversalIterator):
    def __init__(self, resource):
        super(TestResourceTraversalIterator, self).__init__(resource)
        self.events = []

    def on_pre_enter(self):
        self.events.append("on_pre_enter: %s" % self.path)

    def on_enter(self):
        self.events.append("on_enter: %s" % self.path)

    def on_exit(self):
        self.events.append("on_exit: %s" % self.path)


class TestTraversal(object):
    def test_structure(self):
        TEST_STRUCTURE.full_clean()

        resource_iter = TestResourceTraversalIterator(TEST_STRUCTURE)
        resources = ["%s %s %s" % (r, r.name, resource_iter.depth) for r in resource_iter]

        assert [
            'on_enter: ',
                'on_enter: level2',
                'on_exit: level2',
                'on_enter: level2s[a]',
                'on_exit: level2s[a]',
                'on_enter: level2s[b]',
                    'on_enter: level2s[b].level3s{name=e}',
                    'on_exit: level2s[b].level3s{name=e}',
                    'on_enter: level2s[b].level3s{name=f}',
                    'on_exit: level2s[b].level3s{name=f}',
                'on_exit: level2s[b]',
                'on_enter: level2s[c]',
                'on_enter: level2s[c].level3s{name=h}',
                'on_exit: level2s[c].level3s{name=h}',
                'on_exit: level2s[c]',
            'on_exit: ',
        ] == resource_iter.events

        assert [
            'odin.traversal.Level1 resource a 0',
            'odin.traversal.Level2 resource b 1',
            'odin.traversal.Level2 resource c 1',
            'odin.traversal.Level2 resource d 1',
            'odin.traversal.Level3 resource e 2',
            'odin.traversal.Level3 resource f 2',
            'odin.traversal.Level2 resource g 1',
            'odin.traversal.Level3 resource h 2',
        ] == resources

    def test_list_structure(self):
        TEST_STRUCTURE.full_clean()

        resource_iter = TestResourceTraversalIterator(TEST_LIST_STRUCTURE)
        resources = ["%s %s %s" % (r, r.name, resource_iter.depth) for r in resource_iter]

        assert [
            'on_enter: ',
                'on_enter: level2',
                'on_exit: level2',
                'on_enter: level2s[a]',
                'on_exit: level2s[a]',
                'on_enter: level2s[b]',
                    'on_enter: level2s[b].level3s{name=e}',
                    'on_exit: level2s[b].level3s{name=e}',
                    'on_enter: level2s[b].level3s{name=f}',
                    'on_exit: level2s[b].level3s{name=f}',
                'on_exit: level2s[b]',
                'on_enter: level2s[c]',
                'on_enter: level2s[c].level3s{name=h}',
                'on_exit: level2s[c].level3s{name=h}',
                'on_exit: level2s[c]',
            'on_exit: ',
            'on_enter: ',
                'on_enter: level2',
                'on_exit: level2',
                'on_enter: level2s[a]',
                'on_exit: level2s[a]',
                'on_enter: level2s[b]',
                    'on_enter: level2s[b].level3s{name=m}',
                    'on_exit: level2s[b].level3s{name=m}',
                    'on_enter: level2s[b].level3s{name=n}',
                    'on_exit: level2s[b].level3s{name=n}',
                'on_exit: level2s[b]',
                'on_enter: level2s[c]',
                'on_enter: level2s[c].level3s{name=p}',
                'on_exit: level2s[c].level3s{name=p}',
                'on_exit: level2s[c]',
            'on_exit: ',
        ] == resource_iter.events

        assert [
            'odin.traversal.Level1 resource a 0',
            'odin.traversal.Level2 resource b 1',
            'odin.traversal.Level2 resource c 1',
            'odin.traversal.Level2 resource d 1',
            'odin.traversal.Level3 resource e 2',
            'odin.traversal.Level3 resource f 2',
            'odin.traversal.Level2 resource g 1',
            'odin.traversal.Level3 resource h 2',
            'odin.traversal.Level1 resource i 0',
            'odin.traversal.Level2 resource j 1',
            'odin.traversal.Level2 resource k 1',
            'odin.traversal.Level2 resource l 1',
            'odin.traversal.Level3 resource m 2',
            'odin.traversal.Level3 resource n 2',
            'odin.traversal.Level2 resource o 1',
            'odin.traversal.Level3 resource p 2',
        ] == resources


class TestTraversalPath(object):
    def test_parse(self):
        actual = traversal.TraversalPath.parse('level2')
        assert traversal.TraversalPath((traversal.NotSupplied, traversal.NotSupplied, 'level2'),) == actual

        actual = traversal.TraversalPath.parse('level2.name')
        assert traversal.TraversalPath(
            (traversal.NotSupplied, traversal.NotSupplied, 'level2'),
            (traversal.NotSupplied, traversal.NotSupplied, 'name')
        ) == actual

        actual = traversal.TraversalPath.parse('level2s[b].level3s[1].name')
        assert traversal.TraversalPath(
            ('b', traversal.NotSupplied, 'level2s'),
            ('1', traversal.NotSupplied, 'level3s'),
            (traversal.NotSupplied, traversal.NotSupplied, 'name')
        ) == actual

        actual = traversal.TraversalPath.parse('level2s[b].level3s{code=abc}.name')
        assert traversal.TraversalPath(
            ('b', traversal.NotSupplied, 'level2s'),
            ('abc', 'code', 'level3s'),
            (traversal.NotSupplied, traversal.NotSupplied, 'name')
        ) == actual

    def test_add(self):
        actual = traversal.TraversalPath.parse('level2') + 'name'
        assert traversal.TraversalPath.parse('level2.name') == actual

        actual = traversal.TraversalPath.parse('level2s[b]') + traversal.TraversalPath.parse('level3s[1].name')
        assert traversal.TraversalPath.parse('level2s[b].level3s[1].name') == actual

    def test_valid_path(self):
        assert 'a' == traversal.TraversalPath.parse('name').get_value(TEST_STRUCTURE)
        assert 'b' == traversal.TraversalPath.parse('level2.name').get_value(TEST_STRUCTURE)

        r = traversal.TraversalPath.parse('level2s[b].level3s[1]').get_value(TEST_STRUCTURE)
        assert isinstance(r, Level3)
        assert 'f' == r.name

        r = traversal.TraversalPath.parse('level2s[b].level3s{name=f}').get_value(TEST_STRUCTURE)
        assert isinstance(r, Level3)
        assert 'f' == r.name

        r = traversal.TraversalPath.parse('level2s{name=g}.level3s{name=h}').get_value(TEST_STRUCTURE)
        assert isinstance(r, Level3)
        assert 'h' == r.name

    def test_invalid_path(self):
        path = traversal.TraversalPath.parse('level2s[b].level3s[4]')
        pytest.raises(NoMatchError, path.get_value, TEST_STRUCTURE)

        path = traversal.TraversalPath.parse('level2s[b].level3s_sd[1]')
        pytest.raises(InvalidPathError, path.get_value, TEST_STRUCTURE)

        path = traversal.TraversalPath.parse('level2s[d].level3s{name=h}')
        pytest.raises(NoMatchError, path.get_value, TEST_STRUCTURE)

        path = traversal.TraversalPath.parse('level2s{label=not_empty}.level3s{name=h}')
        pytest.raises(MultipleMatchesError, path.get_value, TEST_STRUCTURE)



# -*- coding: utf-8 -*-
from __future__ import absolute_import
from odin import adapters
from odin.utils import field_iter_items
from .resources import *


class TestResourceOptionsAdapter(object):
    def test_repr(self):
        options = adapters.ResourceAdapter(Book())._meta

        assert '<Options Adapter for library.Book>' == repr(options)

    def test_attribute_fields(self):
        options = adapters.ResourceAdapter(Book())._meta

        assert ['fiction'] == [f.name for f in options.attribute_fields]

    def test_elements_fields(self):
        options = adapters.ResourceAdapter(Book())._meta

        assert (
            ['title', 'isbn', 'num_pages', 'rrp', 'genre', 'published', 'authors', 'publisher'] ==
            [f.name for f in options.element_fields]
        )

    def test_attribute_proxying(self):
        options = adapters.ResourceAdapter(Book())._meta

        assert "Book" == options.verbose_name


class TestResourceAdapter(object):
    def test_field_proxy(self):
        target = adapters.ResourceAdapter(Book(title="Foo"))

        assert 'Foo' == target.title

    def test_include_fields(self):
        book = Book(title="Foo", rrp=123.45, num_pages=10, fiction=True)

        target = adapters.ResourceAdapter(book, include=('title', 'rrp'))

        assert ['rrp', 'title'] == sorted(target._meta.field_map.keys())

    def test_exclude_fields(self):
        book = Book(title="Foo", rrp=123.45, num_pages=10, fiction=True)

        target = adapters.ResourceAdapter(book, exclude=('title', 'rrp'))

        assert (
            ['authors', 'fiction', 'genre', 'isbn', 'num_pages', 'published', 'publisher'] ==
            sorted(target._meta.field_map.keys())
        )

    def test_repr(self):
        target = adapters.ResourceAdapter(Book(title="Foo"))

        assert '<ResourceAdapter: library.Book resource adapter>' == repr(target)

    def test_iter(self):
        book = Book(title="Foo", isbn='abc123', rrp=123.45, num_pages=10, fiction=True)
        target = adapters.ResourceAdapter(book)

        actual = list((f.name, str(v)) for f, v in field_iter_items(target))
        assert [
            ('title', 'Foo'),
            ('isbn', 'abc123'),
            ('num_pages', '10'),
            ('rrp', '123.45'),
            ('fiction', 'True'),
            ('genre', 'None'),
            ('published', '[]'),
            ('authors', '[]'),
            ('publisher', 'None')] == actual

    def test_to_dict(self):
        publisher = Publisher()
        book = Book(title="Foo", isbn='abc123', rrp=123.45, num_pages=10, fiction=True, publisher=publisher)
        target = adapters.ResourceAdapter(book)

        assert {
            'authors': [],
            'fiction': True,
            'genre': None,
            'num_pages': 10,
            'published': [],
            'publisher': publisher,
            'rrp': 123.45,
            'isbn': 'abc123',
            'title': 'Foo'
        } == target.to_dict()

    def test_apply_to(self):
        sources = [
            Library(name="Foo"),
            Publisher(name='Bar'),
            Library(name="Eek")
        ]
        result = adapters.ResourceAdapter.apply_to(sources, include=['name'])
        actuals = list(result)

        assert([
            {'name': 'Foo'},
            {'name': 'Bar'},
            {'name': 'Eek'}
        ] == [a.to_dict() for a in actuals])

    def test_curried(self):
        book = Book(title="Foo", isbn='abc123', rrp=123.45, num_pages=10, fiction=True, publisher=Publisher())
        SummaryAdapter = adapters.ResourceAdapter.curry(include=('title', 'rrp', 'fiction'))
        target = SummaryAdapter(book)

        assert ({
            'fiction': True,
            'rrp': 123.45,
            'title': 'Foo'
        } == target.to_dict())



from __future__ import absolute_import, print_function
from humanfriendly import tables
import six
from .base import SummaryBase, resource_reference


class MappingSummary(SummaryBase):
    """
    Produce a text based summary of a mapping
    """
    def __init__(self, mapping, out):
        super(MappingSummary, self).__init__(out)
        self.mapping = mapping

    def render(self):
        mapping = self.mapping
        self.title("{}.{}".format(mapping.__module__, mapping.__name__))
        self.print()
        self.print("From:", resource_reference(mapping.from_obj))
        self.print("To:  ", resource_reference(mapping.to_obj))
        self.print()

        # Build mapping summary
        data = []
        column_names = ('From', 'Action', 'To')
        for from_fields, action, to_fields, to_list, bind, skip_if_none in mapping._mapping_rules:
            # Dereference
            from_fields = from_fields or ['*Assigned*']
            to_fields = to_fields or list()
            row_count = max(len(from_fields), len(to_fields))

            # Normalise
            if len(from_fields) < row_count:
                from_fields += ['' for _ in range(row_count - len(from_fields))]
            if len(to_fields) < row_count:
                to_fields += ['' for _ in range(row_count - len(to_fields))]

            if action:
                if not isinstance(action, six.string_types):
                    action = str(action)
            else:
                action = '-->'

            for idx in range(row_count):
                from_field = from_fields[idx]
                to_field = to_fields[idx]
                if to_list:
                    to_field = '[{}]'.format(to_field)
                if idx == 0:
                    data.append([from_field, action, to_field])
                else:
                    data.append([from_field, '+->', to_field])

        self.print(tables.format_pretty_table(data, column_names))



# -*- coding: utf-8 -*-
from __future__ import absolute_import
import pytest
import odin
from odin.fields import NOT_PROVIDED
from odin.resources import ResourceOptions, build_object_graph
from odin.exceptions import ValidationError
from .resources import Book, Library, Subscriber


class Author(odin.Resource):
    name = odin.StringField()
    country = odin.StringField(null=True)

    def clean(self):
        if self.name == "Bruce" and self.country.startswith("Australia"):
            raise ValidationError("No no no no")

    def clean_country(self, value):
        if value not in ["Australia", "New Zealand"]:
            raise ValidationError("What are ya?")
        return "%s!" % value

    def extra_attrs(self, attrs):
        self.extras = attrs


# Resources for abstract namespace test.
class ResourceA(odin.Resource):
    class Meta:
        abstract = True
        namespace = "example"


class ResourceB(ResourceA):
    class Meta:
        abstract = True


class ResourceC(ResourceB):
    pass


class ResourceD(ResourceC):
    pass


class TestResource(object):
    def test_constructor_kwargs_only(self):
        r = Author(name="Foo")
        assert "Foo" == r.name

    def test_constructor_kwargs_only_with_unknown_field(self):
        with pytest.raises(TypeError):
            Author(name="Foo", age=42)

    def test_constructor_args_only(self):
        r = Author("Foo")
        assert "Foo" == r.name
        assert None == r.country

    def test_constructor_args_excess(self):
        with pytest.raises(TypeError) as ex:
            Author("Foo", "Australia", 42)
        assert "This resource takes 2 positional arguments but 3 where given." == str(ex.value)

    def test_constructor_args_and_kwargs(self):
        r = Author("Foo", country="Australia")
        assert "Foo" == r.name
        assert "Australia" == r.country

    def test_constructor_args_and_kwargs_overrides(self):
        r = Author("Foo", name="Bar", country="Australia")
        assert "Foo" == r.name
        assert "Australia" == r.country

    def test_simple_methods(self):
        r = Author()
        assert "<Author: tests.test_resources.Author resource>" == repr(r)
        assert "tests.test_resources.Author resource" == str(r)

    def test_clean_fields_1(self):
        r = Author(name="Foo")
        r.country = "Australia"

        r.clean_fields()

        assert "Foo" == r.name
        assert "Australia!" == r.country

    def test_clean_fields_2(self):
        r = Author(name="Foo")

        r.clean_fields()

        assert "Foo" == r.name
        assert None == r.country

    def test_clean_fields_3(self):
        r = Author(name="Foo", country="England")

        try:
            r.clean_fields()
        except ValidationError as ve:
            assert ["What are ya?"] == ve.message_dict['country']
        else:
            raise AssertionError("ValidationError not raised.")

    def test_clean_fields_4(self):
        r = Author()

        try:
            r.clean_fields()
        except ValidationError as ve:
            assert ["This field cannot be null."] == ve.message_dict['name']
        else:
            raise AssertionError("ValidationError not raised.")

    def test_full_clean(self):
        r = Author(name="Bruce", country="Australia")

        try:
            r.full_clean()
        except ValidationError as ve:
            assert ["No no no no"] == ve.message_dict['__all__']
        else:
            raise AssertionError("ValidationError not raised.")

    def test_full_clean_exclude(self):
        r = Author(name="Bruce", country="England")

        r.full_clean(exclude=('country',))

        assert "Bruce" == r.name
        assert "England" == r.country

    # Fix for #11
    def test_multiple_abstract_namespaces(self):
        assert 'example.ResourceC' == ResourceC._meta.resource_name

    def test_parents_1(self):
        assert [] == ResourceA._meta.parents
        assert [ResourceA] == ResourceB._meta.parents
        assert [ResourceA, ResourceB] == ResourceC._meta.parents
        assert [ResourceA, ResourceB, ResourceC] == ResourceD._meta.parents


class TestMetaOptions(object):
    def test_invalid_options(self):
        class Meta:
            random_val = 10

        class NewResource(object):
            pass

        target = ResourceOptions(Meta)

        with pytest.raises(TypeError):
            target.contribute_to_class(NewResource, 'etc')


class TestConstructionMethods(object):
    def test_build_object_graph_empty_dict_no_clean(self):
        book = build_object_graph({}, Book, full_clean=False)

        assert dict(
            title=None,
            isbn=None,
            num_pages=None,
            rrp=20.4,
            fiction=None,
            genre=None,
            published=None,
            authors=None,
            publisher=None
        ) == book.to_dict()

    def test_build_object_graph_empty_dict(self):
        with pytest.raises(ValidationError) as ctx:
            build_object_graph({}, Book)

        assert dict(
            title=['This field cannot be null.'],
            isbn=['This field cannot be null.'],
            num_pages=['This field cannot be null.'],
            fiction=['This field cannot be null.'],
            genre=['This field cannot be null.'],
            published=['This field cannot be null.'],
            authors=['List cannot contain null entries.'],
        ) == ctx.value.error_messages

    def test_build_object_graph_from_list(self):
        books = build_object_graph([dict(
            title="Book1"
        ), dict(
            title="Book2"
        )], Book, full_clean=False)

        assert [dict(
            title="Book1",
            isbn=None,
            num_pages=None,
            rrp=20.4,
            fiction=None,
            genre=None,
            published=None,
            authors=None,
            publisher=None
        ), dict(
            title="Book2",
            isbn=None,
            num_pages=None,
            rrp=20.4,
            fiction=None,
            genre=None,
            published=None,
            authors=None,
            publisher=None
        )] == [book.to_dict() for book in books]

    def test_build_nested_objects(self):
        subscribers = [
            {'name': 'John Smith', 'address': 'Oak Lane 1234'},
            {'name': 'Johnny Smith', 'address': 'Oak Lane 1235'}]

        library = {
            'name': 'John Smith Library',
            'subscribers': subscribers
        }

        expected = sorted(build_object_graph(subscribers, resource=Subscriber, full_clean=False), key=lambda s: s.name)
        actual = sorted(build_object_graph(library, resource=Library, full_clean=False).subscribers, key=lambda s: s.name)

        assert actual == expected

    def test_build_nested_objects_with_polymorphism(self):
        books = [{'title': "Book1", "isbn": "abc-123", 'num_pages': 1, 'rrp': 20.4, 'fiction': True, 'genre': 'sci-fi', 'published': [],
                  'authors': [], 'publisher': None, '$': 'tests.resources.Book'},
                 {'title': "Book2", "isbn": "def-456", 'num_pages': 1, 'rrp': 20.4, 'fiction': True, 'genre': 'sci-fi', 'published': [],
                  'authors': [], 'publisher': None, '$': 'tests.resources.Book'}]

        library = {
            'name': 'John Smith Library',
            'books': books
        }

        expected = sorted(build_object_graph(books, full_clean=False), key=lambda s: s.title)
        actual = sorted(build_object_graph(library, resource=Library, full_clean=False).books, key=lambda s: s.title)

        assert actual == expected

    def test_create_resource_from_dict_with_default_to_not_supplied(self):
        book = build_object_graph({
            "title": "Foo",
            "num_pages": 42
        }, Book, full_clean=False, default_to_not_supplied=True)

        assert dict(
            title="Foo",
            isbn=NOT_PROVIDED,
            num_pages=42,
            rrp=NOT_PROVIDED,
            fiction=NOT_PROVIDED,
            genre=NOT_PROVIDED,
            published=NOT_PROVIDED,
            authors=NOT_PROVIDED,
            publisher=NOT_PROVIDED
        ) == book.to_dict()



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function
from odin import exceptions
from odin.fields import Field, ScalarField
from odin.validators import EMPTY_VALUES
from .datatypes import latitude, longitude, latlng, point

__all__ = ('LatitudeField', 'LongitudeField', 'LatLngField', 'PointField')


class LatitudeField(ScalarField):
    """
    Field that contains a latitude value.
    """
    default_error_messages = {
        'invalid': "'%s' value must be a latitude.",
    }
    data_type_name = "Latitude"

    def to_python(self, value):
        if value in EMPTY_VALUES:
            return
        try:
            return latitude(value)
        except (ValueError, TypeError):
            msg = self.error_messages['invalid'] % value
            raise exceptions.ValidationError(msg)


class LongitudeField(ScalarField):
    """
    Field that contains a longitude value.
    """
    default_error_messages = {
        'invalid': "'%s' value must be a longitude.",
    }
    data_type_name = "Longitude"

    def to_python(self, value):
        if value in EMPTY_VALUES:
            return
        try:
            return longitude(value)
        except (ValueError, TypeError):
            msg = self.error_messages['invalid'] % value
            raise exceptions.ValidationError(msg)


class LatLngField(Field):
    """
    Field that contains a lat/long pair.
    """
    default_error_messages = {
        'invalid': "'%s' value must be a (latitude, longitude).",
    }
    data_type_name = "LatLng"

    def to_python(self, value):
        if value in EMPTY_VALUES:
            return
        try:
            return latlng(value)
        except (ValueError, TypeError):
            msg = self.error_messages['invalid'] % value
            raise exceptions.ValidationError(msg)


class PointField(Field):
    """
    Field that contains a point in cartesian space. This can be either 2D (on a plain) or 3D (includes a z-axis).
    """
    default_error_messages = {
        'invalid': "'%s' value must be a point in 2D or 3D cartesian space.",
    }
    data_type_name = "Point"

    def to_python(self, value):
        if value in EMPTY_VALUES:
            return
        try:
            return point(value)
        except (ValueError, TypeError):
            msg = self.error_messages['invalid'] % value
            raise exceptions.ValidationError(msg)



# -*- coding: utf-8 -*-
import pytest
from odin import validators
from odin.exceptions import ValidationError


class TestValidator(object):
    def test_regex_validator(self):
        target = validators.RegexValidator(r'^[a-z]{3}$', "Please enter 3 alpha characters.", "match_chars")

        assert "Please enter 3 alpha characters." == target.message
        assert "match_chars" == target.code

        target("abc")
        target("cba")
        pytest.raises(ValidationError, target, "a")
        pytest.raises(ValidationError, target, "abcd")
        pytest.raises(ValidationError, target, "123")

    def test_url(self):
        validators.validate_url('http://www.djangoproject.com/')
        validators.validate_url('http://localhost/')
        validators.validate_url('http://example.com/')
        validators.validate_url('http://www.example.com/')
        validators.validate_url('http://www.example.com:8000/test')
        validators.validate_url('http://valid-with-hyphens.com/')
        validators.validate_url('http://subdomain.example.com/')
        validators.validate_url('http://200.8.9.10/')
        validators.validate_url('http://200.8.9.10:8000/test')
        validators.validate_url('http://valid-----hyphens.com/')
        validators.validate_url('http://example.com?something=value')
        validators.validate_url('http://example.com/index.php?something=value&another=value2')
        validators.validate_url('https://example.com/')
        validators.validate_url('ftp://example.com/')
        validators.validate_url('ftps://example.com/')
        validators.validate_url('http://savage.company/')
    
        pytest.raises(ValidationError, validators.validate_url, 'foo')
        pytest.raises(ValidationError, validators.validate_url, 'http://')
        pytest.raises(ValidationError, validators.validate_url, 'http://example')
        pytest.raises(ValidationError, validators.validate_url, 'http://example.')
        pytest.raises(ValidationError, validators.validate_url, 'http://.com')
        pytest.raises(ValidationError, validators.validate_url, 'http://invalid-.com')
        pytest.raises(ValidationError, validators.validate_url, 'http://-invalid.com')
        pytest.raises(ValidationError, validators.validate_url, 'http://inv-.alid-.com')
        pytest.raises(ValidationError, validators.validate_url, 'http://inv-.-alid.com')

    def test_max_value_validator(self):
        target = validators.MaxValueValidator(10)

        target(10)
        target(1)
        target(-10)
        pytest.raises(ValidationError, target, 11)

    def test_min_value_validator(self):
        target = validators.MinValueValidator(10)

        target(10)
        pytest.raises(ValidationError, target, 1)
        pytest.raises(ValidationError, target, -10)
        target(11)

    def test_length_validator(self):
        target = validators.LengthValidator(10)
        target("1234567890")
        pytest.raises(ValidationError, target, "123456789")
        pytest.raises(ValidationError, target, "12345678901")

    def test_max_length_validator(self):
        target = validators.MaxLengthValidator(10)

        target("123457890")
        target("12345")
        target("")
        pytest.raises(ValidationError, target, "12345678901")

    def test_min_length_validator(self):
        target = validators.MinLengthValidator(10)

        pytest.raises(ValidationError, target, "123457890")
        pytest.raises(ValidationError, target, "12345")
        pytest.raises(ValidationError, target, "")
        target("12345678901")


class TestSimpleValidator(object):
    def test_method(self):
        def reflect(v):
            return v

        validator = validators.simple_validator(reflect)

        pytest.raises(ValidationError, validator, False)
        validator(True)

    def test_decorator(self):

        @validators.simple_validator()
        def reflect_validator(v):
            return v

        pytest.raises(ValidationError, reflect_validator, False)
        reflect_validator(True)



from __future__ import absolute_import
import pytest
import os
from odin import filtering
from odin.traversal import TraversalPath
from odin.codecs import json_codec


# Open library fixture
with open(os.path.join(os.path.dirname(__file__), 'fixtures', 'library.json')) as f:
    library = json_codec.load(f)

first_book = TraversalPath.parse('books[0]').get_value(library)


class TestFilter(object):
    def test_matches_top_level_resource(self):
        flt = filtering.Equal('title', "Consider Phlebas")
        assert flt(first_book)

    def test_matches_multiple_matches_resource(self):
        # Less than fiction $20 with a single author and not fantasy
        flt = filtering.And(
            filtering.Equal('fiction', True),
            filtering.LessThan('rrp', 20),
            filtering.Equal('authors', 1, len),
            filtering.NotEqual('genre', 'fantasy'),
        )
        assert flt(first_book)

    def test_matches_sub_resource(self):
        flt = filtering.Equal('publisher.name', 'Macmillan')
        assert flt(first_book)

    def test_find_sub_resources(self):
        flt = filtering.Equal('fiction', True)
        # Some fiction?
        assert flt.any(library.books)
        # All fiction
        assert not flt.all(library.books)

    def test_resource_field_does_not_exist(self):
        flt = filtering.Equal('title', 'Public Library')
        assert not flt(library)


class TestFilterChain(object):
    def test_empty_description(self):
        flt = filtering.And()
        assert '' == str(flt)

    def test_description(self):
        flt = filtering.And(
            filtering.Equal('fiction', True),
            filtering.LessThan('rrp', 20),
        )
        assert '(fiction == True AND rrp < 20)' == str(flt)

    def test_combining_same_type(self):
        flt1 = filtering.And(
            filtering.Equal('fiction', True),
            filtering.LessThan('rrp', 20),
        )
        flt2 = filtering.And(
            filtering.Equal('authors', 1, len),
            filtering.NotEqual('genre', 'fantasy'),
        )
        flt = flt1 + flt2

        assert "(fiction == True AND rrp < 20 AND len(authors) == 1 AND genre != 'fantasy')" == str(flt)

    def test_combine_with_comparison(self):
        flt1 = filtering.And(
            filtering.Equal('fiction', True),
            filtering.LessThan('rrp', 20),
        )
        flt = flt1 + filtering.Equal('authors', 1, len)

        assert "(fiction == True AND rrp < 20 AND len(authors) == 1)" == str(flt)

    def test_combine_with_other_type(self):
        flt1 = filtering.And(
            filtering.Equal('fiction', True),
            filtering.LessThan('rrp', 20),
        )
        pytest.raises(TypeError, lambda: flt1 + 'abc')


class TestFilterComparison(object):
    def test_description(self):
        comp = filtering.FilterComparison('foo', 'bar')
        comp.operator_symbols = ['%']
        assert "foo % 'bar'" == str(comp)
        comp = filtering.FilterComparison('foo', 42)
        comp.operator_symbols = ['%']
        assert 'foo % 42' == str(comp)
        comp = filtering.FilterComparison('foo', 'bar', any)
        comp.operator_symbols = ['%']
        assert "any(foo) % 'bar'" == str(comp)

    def test_equal_string(self):
        comp = filtering.Equal('foo', 'bar')
        assert comp.compare('bar')
        assert not comp.compare('eek')
        assert "foo == 'bar'" == str(comp)

    def test_equal_integer(self):
        comp = filtering.Equal('foo', 5)
        assert comp.compare(5)
        assert not comp.compare(6)
        assert 'foo == 5' == str(comp)

    def test_not_equal(self):
        comp = filtering.NotEqual('foo', 'bar')
        assert comp.compare('eek')
        assert not comp.compare('bar')
        assert "foo != 'bar'" == str(comp)

    def test_less_than(self):
        comp = filtering.LessThan('foo', 5)
        assert comp.compare(4)
        assert not comp.compare(5)
        assert not comp.compare(6)
        assert 'foo < 5' == str(comp)

    def test_less_than_or_equal(self):
        comp = filtering.LessThanOrEqual('foo', 5)
        assert comp.compare(4)
        assert comp.compare(5)
        assert not comp.compare(6)
        assert 'foo <= 5' == str(comp)

    def test_greater_than(self):
        comp = filtering.GreaterThan('foo', 5)
        assert not comp.compare(4)
        assert not comp.compare(5)
        assert comp.compare(6)
        assert 'foo > 5' == str(comp)

    def test_greater_than_or_equal(self):
        comp = filtering.GreaterThanOrEqual('foo', 5)
        assert not comp.compare(4)
        assert comp.compare(5)
        assert comp.compare(6)
        assert 'foo >= 5' == str(comp)



from setuptools import setup, find_packages
from codecs import open
from os import path

here = path.abspath(path.dirname(__file__))

# Get the long description from the README file
with open(path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='odin',

    version='0.8.2',

    description='Data-structure definition/validation/traversal, mapping and serialisation toolkit for Python',
    long_description=long_description,

    url='https://github.com/python-odin/odin',

    author='Tim Savage',
    author_email='tim@savage.company',

    license='BSD',

    classifiers=[
        'Development Status :: 5 - Production/Stable',

        'Intended Audience :: Developers',

        'License :: OSI Approved :: BSD License',

        'Operating System :: OS Independent',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
        'Topic :: Software Development :: Libraries :: Python Modules',

        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: Implementation :: PyPy',
    ],

    keywords='data-structure validation data-mapping',

    packages=find_packages(),

    install_requires=['six'],

    extras_require={
        # Additional codecs
        'yaml': ['pyyaml'],
        'msgpack': ['msgpack-python'],

        # Documentation generation
        'doc_gen': ["jinja2>=2.7"],
        # Pint integration
        'pint': ["pint"],
        # Filter Query
        'filter_query': ["ply"],
        # Inspect
        'inspect': ['humanfriendly'],
    },

    obsoletes=['jsrn'],
)



# -*- coding: utf-8 -*-


class VirtualField(object):
    """
    Base class for virtual fields. A virtual fields is treated like any other field during encoding/decoding (provided
    it can be written to).
    """
    # These track each time a VirtualField instance is created. Used to retain order.
    creation_counter = 0
    data_type_name = None

    def __init__(self, verbose_name=None, verbose_name_plural=None, name=None, data_type_name=None, doc_text='',
                 is_attribute=False):
        """
        Initialisation of virtual field

        :param verbose_name: Display name of field.
        :param verbose_name_plural: Plural display name of field.
        :param name: Name of the serialised field.
        :param data_type_name: A name for the data type this field returns (used for generating documentation)
        :param doc_text: Documentation for the field, replaces help text
        :param is_attribute: Special flag for codecs that support attributes on nodes (ie XML)
        """
        self.verbose_name, self.verbose_name_plural = verbose_name, verbose_name_plural
        self.name = name
        self.doc_text = doc_text
        self.data_type_name = data_type_name
        self.is_attribute = is_attribute

        self.creation_counter = VirtualField.creation_counter
        VirtualField.creation_counter += 1

    def __hash__(self):
        return self.creation_counter

    def __get__(self, instance, owner):
        raise NotImplementedError

    def __set__(self, instance, value):
        raise AttributeError("Read only")

    def __repr__(self):
        """
        Displays the module, class and name of the field.
        """
        path = '%s.%s' % (self.__class__.__module__, self.__class__.__name__)
        name = getattr(self, 'name', None)
        if name is not None:
            return '<%s: %s>' % (path, name)
        return '<%s>' % path

    def set_attributes_from_name(self, attname):
        if not self.name:
            self.name = attname
        self.attname = attname
        if self.verbose_name is None and self.name:
            self.verbose_name = self.name.replace('_', ' ')
        if self.verbose_name_plural is None and self.verbose_name:
            self.verbose_name_plural = "%ss" % self.verbose_name

    def contribute_to_class(self, cls, name):
        self.set_attributes_from_name(name)
        self.resource = cls
        cls._meta.add_virtual_field(self)

        setattr(cls, name, self)

    def prepare(self, value):
        """
        Prepare a value for serialisation.

        :param value:
        :return:
        """
        return value

    def value_from_object(self, obj):
        """
        Returns the value of this field in the given resource instance.
        """
        return getattr(obj, self.attname)


class ConstantField(VirtualField):
    """
    A field that provides a constant value.
    """
    def __init__(self, value, *args, **kwargs):
        super(ConstantField, self).__init__(*args, **kwargs)
        self.value = value

    def __get__(self, instance, owner):
        return self.value


class CalculatedField(VirtualField):
    """
    A field whose that is calculated by an expression.

    The expression should accept a single "self" parameter that is a Resource instance.
    """
    def __init__(self, expr, *args, **kwargs):
        assert callable(expr)
        super(CalculatedField, self).__init__(*args, **kwargs)
        self.expr = expr

    def __get__(self, instance, owner):
        return self.expr(instance)


def calculated_field(method=None, **kwargs):
    """
    Converts an instance method into a calculated field.
    """
    def inner(expr):
        return CalculatedField(expr, **kwargs)
    return inner if method is None else inner(method)
